@@ -45,9 +45,10 @@
 // the source.
 //
 
-#include <ctype.h>
-#include <math.h>
-#include <float.h>
+#include <cctype>
+#include <cstdlib>
+#include <cmath>
+#include <cfloat>
 
 #include "sysc/datatypes/fx/sc_fxval.h"
 
@@ -207,7 +208,7 @@ print_dec( scfx_string& s, scfx_ieee_double id, int w_prefix, sc_fmt fmt )
     // split 'id' into its integer and fractional part
 
     double int_part;
-    double frac_part = modf( static_cast<double>( id ), &int_part );
+    double frac_part = std::modf( static_cast<double>( id ), &int_part );
 
     int i;
 
@@ -218,7 +219,7 @@ print_dec( scfx_string& s, scfx_ieee_double id, int w_prefix, sc_fmt fmt )
     
     if( int_part != 0.0 )
     {
-	int_digits = (int) ceil( log10( int_part + 1.0 ) );
+	int_digits = (int) std::ceil( std::log10( int_part + 1.0 ) );
 
 	int len = s.length();
 	s.append( int_digits );
@@ -227,7 +228,7 @@ print_dec( scfx_string& s, scfx_ieee_double id, int w_prefix, sc_fmt fmt )
 
 	for( i = int_digits + len - 1; i >= len; i-- )
 	{
-	    unsigned int remainder = (unsigned int) fmod( int_part, 10.0 );
+	    unsigned int remainder = (unsigned int) std::fmod( int_part, 10.0 );
 	    s[i] = static_cast<char>( '0' + remainder );
 	    
 	    if( zero_digits )
@@ -263,9 +264,9 @@ print_dec( scfx_string& s, scfx_ieee_double id, int w_prefix, sc_fmt fmt )
 
 	bool zero_digits = ( int_digits == 0 && fmt != SC_F );
 
-	frac_zeros = (int) floor( - log10( frac_part + DBL_EPSILON ) );
+	frac_zeros = (int) std::floor( - std::log10( frac_part + DBL_EPSILON ) );
 
-	frac_part *= pow( 10.0, frac_zeros );
+	frac_part *= std::pow( 10.0, frac_zeros );
 	
 	frac_digits = frac_zeros;
 	if( ! zero_digits )
@@ -415,12 +416,14 @@ print_other( scfx_string& s, const scfx_ieee_double& id, sc_numrep numrep,
 	    step = 4;
 	    break;
 	default:
-	    step = 0;
+            SC_REPORT_FATAL( sc_core::SC_ID_ASSERTION_FAILED_
+                           , "unexpected sc_numrep" );
+            sc_core::sc_abort();
     }
 
-    msb = (int) ceil( double( msb + 1 ) / step ) * step - 1;
+    msb = (int) std::ceil( double( msb + 1 ) / step ) * step - 1;
 
-    lsb = (int) floor( double( lsb ) / step ) * step;
+    lsb = (int) std::floor( double( lsb ) / step ) * step;
 
     if( msb < 0 )
     {
@@ -784,7 +787,7 @@ sc_fxval_fast::from_string( const char* s )
     {
 	for( const char *e = end + 2; *e; e ++ )
 	    SCFX_FAIL_IF_( ! scfx_is_digit( *e, SC_DEC ) );
-	exponent = atoi( end + 1 );
+	exponent = std::atoi( end + 1 );
     }
 
     //
@@ -869,7 +872,7 @@ sc_fxval_fast::from_string( const char* s )
 	}
     }
 
-    double exp = ( exponent != 0 ) ? pow( (double) base, (double) exponent )
+    double exp = ( exponent != 0 ) ? std::pow( (double) base, (double) exponent )
 	                           : 1;
 
     return ( sign * ( integer + fraction ) * exp );
