@@ -56,17 +56,42 @@
 // source.
 //
 
-#include <ctype.h>
+#include <cctype>
 #include <cstdio>
-#include <string.h>
+#include <cstring>
+#include <sstream>
 
 #include "sysc/datatypes/int/sc_int_ids.h"
 #include "sysc/datatypes/int/sc_nbutils.h"
 #include "sysc/kernel/sc_macros.h"
 
 
-namespace sc_dt
+namespace sc_dt {
+
+// only used within vec_from_str (non-standard, deprecated)
+static inline void
+is_valid_base(sc_numrep base)
 {
+  switch (base) {
+    case SC_NOBASE: case SC_BIN:
+    case SC_OCT: case SC_DEC:
+    case SC_HEX:
+        break;
+    case SC_BIN_US: case SC_BIN_SM:
+    case SC_OCT_US: case SC_OCT_SM:
+    case SC_HEX_US: case SC_HEX_SM:
+    case SC_CSD:
+      SC_REPORT_ERROR( sc_core::SC_ID_NOT_IMPLEMENTED_,
+                       "is_valid_base( sc_numrep base ) : "
+                       "bases SC_CSD, or ending in _US and _SM are not supported" );
+      break;
+    default:
+      std::stringstream msg;
+      msg << "is_valid_base( sc_numrep base ) : base = " << base
+          << " is not valid";
+      SC_REPORT_ERROR( sc_core::SC_ID_VALUE_NOT_VALID_, msg.str().c_str() );
+  }
+}
 
 // ----------------------------------------------------------------------------
 //  ENUM : sc_numrep
@@ -160,7 +185,7 @@ fsm_move(char c, small_type &b, small_type &s, small_type &state)
 
   default:
     // Any other state is not possible.
-    assert((0 <= state) && (state <= 3));
+    sc_assert((0 <= state) && (state <= 3));
 
   } // switch
 
@@ -173,12 +198,12 @@ fsm_move(char c, small_type &b, small_type &s, small_type &state)
 // pointer to the first char after the point where b and s are
 // determined or where the end of v is reached. The input string v has
 // to be null terminated.
-const char 
-*get_base_and_sign(const char *v, small_type &b, small_type &s)
+const char *
+get_base_and_sign(const char *v, small_type &b, small_type &s)
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert(v != NULL);
+  sc_assert(v != NULL);
 #endif
 
   const small_type STATE_START = 0;
@@ -204,22 +229,19 @@ const char
     }
   }
 
-#ifdef DEBUG_SYSTEMC
   // Test to see if the above loop executed more than it should
   // have. The max number of skipped chars is equal to the length of
   // the longest format specifier, e.g., "-0x".
-  assert(nskip <= 3);
-#endif
+  sc_assert(nskip <= 3);
 
   v += nskip;
 
   // Handles empty strings or strings without any digits after the
   // base or base and sign specifier.
   if (*v == '\0') { 
-      char msg[BUFSIZ];
-      std::sprintf( msg,
-	       "get_base_and_sign( const char* v, small_type&, small_type& ) : "
-	       "v = \"\" is not valid" );
+      static const char msg[]
+         = "get_base_and_sign( const char* v, small_type&, small_type& ) : "
+           "v = \"\" is not valid";
       SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_, msg );
   }
 
@@ -255,10 +277,12 @@ void parse_binary_bits(
     if( src_p == 0 ) {
         SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_,
                          "character string is zero" );
+        return;
     }
     if( *src_p == 0 ) {
         SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_,
                          "character string is empty" );
+        return;
     }
 
 
@@ -298,7 +322,6 @@ void parse_binary_bits(
         if ( src_i < 0 ) 
         {
             src_n += BITS_PER_DIGIT;
-            src_i = 0;
             data = 0;
             ctrl = 0;
             for ( src_i = 0; src_i < src_n; src_i++ )
@@ -315,10 +338,10 @@ void parse_binary_bits(
                   case '0':                  break;
                   default:
                     {
-                        char msg[BUFSIZ];
-                        std::sprintf( msg, "character string '%s' is not valid", 
-                        src_p );
-                        SC_REPORT_ERROR(sc_core::SC_ID_CONVERSION_FAILED_, msg);
+                        std::stringstream msg;
+                        msg << "character string '" << src_p << "' is not valid";
+                        SC_REPORT_ERROR(sc_core::SC_ID_CONVERSION_FAILED_, msg.str().c_str() );
+                        return;
                     }
                     break;
                 }
@@ -347,10 +370,10 @@ void parse_binary_bits(
               case '0':                  break;
               default:
                 {
-                    char msg[BUFSIZ];
-                    std::sprintf( msg, "character string '%s' is not valid", 
-                    src_p );
-                    SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_, msg );
+                    std::stringstream msg;
+                    msg << "character string '" << src_p << "' is not valid";
+                    SC_REPORT_ERROR(sc_core::SC_ID_CONVERSION_FAILED_, msg.str().c_str() );
+                    return;
                 }
                 break;
             }
@@ -360,7 +383,7 @@ void parse_binary_bits(
         src_n = src_n - BITS_PER_DIGIT;
     }
 }
-        
+
 
 //------------------------------------------------------------------------------
 //"parse_hex_bits"
@@ -390,10 +413,12 @@ void parse_hex_bits(
     if( src_p == 0 ) {
         SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_,
                          "character string is zero" );
+        return;
     }
     if( *src_p == 0 ) {
         SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_,
                          "character string is empty" );
+        return;
     }
 
 
@@ -433,7 +458,6 @@ void parse_hex_bits(
         if ( src_i < 0 ) 
         {
             src_n += 8;
-            src_i = 0;
             data = 0;
             ctrl = 0;
             for ( src_i = 0; src_i < src_n; src_i++ )
@@ -470,10 +494,10 @@ void parse_hex_bits(
                   case 'z': ctrl = ctrl | 15; break;
                   default:
                     {
-                        char msg[BUFSIZ];
-                        std::sprintf( msg, "character string '%s' is not valid", 
-                        src_p );
-                        SC_REPORT_ERROR(sc_core::SC_ID_CONVERSION_FAILED_, msg);
+                        std::stringstream msg;
+                        msg << "character string '" << src_p << "' is not valid";
+                        SC_REPORT_ERROR(sc_core::SC_ID_CONVERSION_FAILED_, msg.str().c_str() );
+                        return;
                     }
                     break;
                 }
@@ -522,10 +546,10 @@ void parse_hex_bits(
 	      case 'z': ctrl = ctrl | 15; break;
               default:
                 {
-                    char msg[BUFSIZ];
-                    std::sprintf( msg, "character string '%s' is not valid", 
-                    src_p );
-                    SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_, msg );
+                    std::stringstream msg;
+                    msg << "character string '" << src_p << "' is not valid";
+                    SC_REPORT_ERROR(sc_core::SC_ID_CONVERSION_FAILED_, msg.str().c_str() );
+                    return;
                 }
                 break;
             }
@@ -550,8 +574,8 @@ vec_from_str(int unb, int und, sc_digit *u,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((unb > 0) && (und > 0) && (u != NULL));
-  assert(v != NULL);
+  sc_assert((unb > 0) && (und > 0) && (u != NULL));
+  sc_assert(v != NULL);
 #endif
 
   is_valid_base(base);
@@ -564,12 +588,11 @@ vec_from_str(int unb, int und, sc_digit *u,
     if (b == NB_DEFAULT_BASE)
       b = base;
     else {
-	char msg[BUFSIZ];
-	std::sprintf( msg,
-		 "vec_from_str( int, int, sc_digit*, const char*, sc_numrep base ) : "
-		 "base = %s does not match the default base",
-		 to_string( base ).c_str() );
-	SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_, msg );
+        std::stringstream msg;
+        msg << "vec_from_str( int, int, sc_digit*, const char*, sc_numrep base ) : "
+            << "base = " << base << " does not match the default base";
+        SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_, msg.str().c_str() );
+        return 0;
     }
   }
 
@@ -587,16 +610,15 @@ vec_from_str(int unb, int und, sc_digit *u,
         val = toupper(c) - 'A' + 10;
       else
         val = c - '0';
-      
+
       if (val >= b) {
-	  char msg[BUFSIZ];
-	  std::sprintf( msg,
-		   "vec_from_str( int, int, sc_digit*, const char*, sc_numrep base ) : "
-		   "'%c' is not a valid digit in base %d",
-		   *v, b );
-	  SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_, msg );
+          std::stringstream msg;
+          msg << "vec_from_str( int, int, sc_digit*, const char*, sc_numrep base ) : "
+              << "'" << *v << "' is not a valid digit in base " << b;
+          SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_, msg.str().c_str() );
+          return 0;
       }
-      
+
       // digit = digit * b + val;
       vec_mul_small_on(und, u, b);
       
@@ -605,12 +627,11 @@ vec_from_str(int unb, int und, sc_digit *u,
 
     }
     else {
-	char msg[BUFSIZ];
-	std::sprintf( msg,
-		 "vec_from_str( int, int, sc_digit*, const char*, sc_numrep base ) : "
-		 "'%c' is not a valid digit in base %d",
-		 *v, b );
-	SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_, msg );
+        std::stringstream msg;
+        msg << "vec_from_str( int, int, sc_digit*, const char*, sc_numrep base ) : "
+            << "'" << *v << "' is not a valid digit in base " << b;
+        SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_, msg.str().c_str() );
+        return 0;
     }
   }
 
@@ -632,10 +653,10 @@ vec_add(int ulen, const sc_digit *u,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
-  assert((vlen > 0) && (v != NULL));
-  assert(w != NULL);
-  assert(ulen >= vlen);
+  sc_assert((ulen > 0) && (u != NULL));
+  sc_assert((vlen > 0) && (v != NULL));
+  sc_assert(w != NULL);
+  sc_assert(ulen >= vlen);
 #endif
 
   const sc_digit *uend = (u + ulen);
@@ -676,9 +697,9 @@ vec_add_on(int ulen, sc_digit *ubegin,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (ubegin != NULL));
-  assert((vlen > 0) && (v != NULL));
-  assert(ulen >= vlen);
+  sc_assert((ulen > 0) && (ubegin != NULL));
+  sc_assert((vlen > 0) && (v != NULL));
+  sc_assert(ulen >= vlen);
 #endif
 
   sc_digit *u = ubegin;
@@ -725,9 +746,9 @@ vec_add_on2(int ulen, sc_digit *ubegin,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (ubegin != NULL));
-  assert((vlen > 0) && (v != NULL));
-  assert(ulen < vlen);
+  sc_assert((ulen > 0) && (ubegin != NULL));
+  sc_assert((vlen > 0) && (v != NULL));
+  sc_assert(ulen < vlen);
 #endif
 
   sc_digit *u = ubegin;
@@ -761,8 +782,8 @@ vec_add_small(int ulen, const sc_digit *u,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
-  assert(w != NULL);
+  sc_assert((ulen > 0) && (u != NULL));
+  sc_assert(w != NULL);
 #endif
 
   const sc_digit *uend = (u + ulen);
@@ -795,7 +816,7 @@ vec_add_small_on(int ulen, sc_digit *u, sc_digit v)
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
+  sc_assert((ulen > 0) && (u != NULL));
 #endif
 
   int i = 0;
@@ -827,10 +848,10 @@ vec_sub(int ulen, const sc_digit *u,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
-  assert((vlen > 0) && (v != NULL));
-  assert(w != NULL);
-  assert(ulen >= vlen);
+  sc_assert((ulen > 0) && (u != NULL));
+  sc_assert((vlen > 0) && (v != NULL));
+  sc_assert(w != NULL);
+  sc_assert(ulen >= vlen);
 #endif
 
   const sc_digit *uend = (u + ulen);
@@ -853,7 +874,7 @@ vec_sub(int ulen, const sc_digit *u,
   }
 
 #ifdef DEBUG_SYSTEMC
-  assert(borrow == 0);
+  sc_assert(borrow == 0);
 #endif
 
   // Copy the rest of u to the result.
@@ -871,9 +892,9 @@ vec_sub_on(int ulen, sc_digit *ubegin,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (ubegin != NULL));
-  assert((vlen > 0) && (v != NULL));
-  assert(ulen >= vlen);
+  sc_assert((ulen > 0) && (ubegin != NULL));
+  sc_assert((vlen > 0) && (v != NULL));
+  sc_assert(ulen >= vlen);
 #endif
 
   sc_digit *u = ubegin;
@@ -897,7 +918,7 @@ vec_sub_on(int ulen, sc_digit *ubegin,
   }
 
 #ifdef DEBUG_SYSTEMC
-  assert(borrow == 0);
+  sc_assert(borrow == 0);
 #endif
 
 }
@@ -911,8 +932,8 @@ vec_sub_on2(int ulen, sc_digit *ubegin,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (ubegin != NULL));
-  assert((vlen > 0) && (v != NULL));
+  sc_assert((ulen > 0) && (ubegin != NULL));
+  sc_assert((vlen > 0) && (v != NULL));
 #endif
 
   sc_digit *u = ubegin;
@@ -945,8 +966,8 @@ vec_sub_small(int ulen, const sc_digit *u,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert(ulen > 0);
-  assert(u != NULL);
+  sc_assert(ulen > 0);
+  sc_assert(u != NULL);
 #endif
 
   const sc_digit *uend = (u + ulen);
@@ -964,7 +985,7 @@ vec_sub_small(int ulen, const sc_digit *u,
   }
 
 #ifdef DEBUG_SYSTEMC
-  assert(borrow == 0);
+  sc_assert(borrow == 0);
 #endif
 
   // Copy the rest of u to the result.
@@ -980,7 +1001,7 @@ vec_sub_small_on(int ulen, sc_digit *u, sc_digit v)
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
+  sc_assert((ulen > 0) && (u != NULL));
 #endif
 
   for (int i = 0; i < ulen; ++i) {
@@ -990,7 +1011,7 @@ vec_sub_small_on(int ulen, sc_digit *u, sc_digit v)
   }
 
 #ifdef DEBUG_SYSTEMC
-  assert(v == 0);
+  sc_assert(v == 0);
 #endif
 
 }
@@ -1059,9 +1080,9 @@ vec_mul(int ulen, const sc_digit *u,
   */
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
-  assert((vlen > 0) && (vbegin != NULL));
-  assert(wbegin != NULL);
+  sc_assert((ulen > 0) && (u != NULL));
+  sc_assert((vlen > 0) && (vbegin != NULL));
+  sc_assert(wbegin != NULL);
 #endif
 
 #define prod_h carry
@@ -1077,7 +1098,7 @@ vec_mul(int ulen, const sc_digit *u,
 
 #ifdef DEBUG_SYSTEMC
     // The overflow bits must be zero.
-    assert(u_h == (u_h & HALF_DIGIT_MASK));
+    sc_assert(u_h == (u_h & HALF_DIGIT_MASK));
 #endif
 
     sc_digit carry = 0;
@@ -1095,7 +1116,7 @@ vec_mul(int ulen, const sc_digit *u,
 
 #ifdef DEBUG_SYSTEMC
       // The overflow bits must be zero.
-      assert(v_h == (v_h & HALF_DIGIT_MASK));
+      sc_assert(v_h == (v_h & HALF_DIGIT_MASK));
 #endif
 
       sc_digit prod_l = (*w) + u_l * v_l + low_half(carry);
@@ -1124,9 +1145,9 @@ vec_mul_small(int ulen, const sc_digit *u,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
-  assert(w != NULL);
-  assert((0 < v) && (v < HALF_DIGIT_RADIX));
+  sc_assert((ulen > 0) && (u != NULL));
+  sc_assert(w != NULL);
+  sc_assert((0 < v) && (v < HALF_DIGIT_RADIX));
 #endif
 
 #define prod_h carry
@@ -1141,7 +1162,7 @@ vec_mul_small(int ulen, const sc_digit *u,
 
 #ifdef DEBUG_SYSTEMC
     // The overflow bits must be zero.
-    assert(high_half(u_AB) == high_half_masked(u_AB));
+    sc_assert(high_half(u_AB) == high_half_masked(u_AB));
 #endif
 
     sc_digit prod_l = v * low_half(u_AB) + low_half(carry);
@@ -1167,8 +1188,8 @@ vec_mul_small_on(int ulen, sc_digit *u, sc_digit v)
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
-  assert((0 < v) && (v < HALF_DIGIT_RADIX));
+  sc_assert((ulen > 0) && (u != NULL));
+  sc_assert((0 < v) && (v < HALF_DIGIT_RADIX));
 #endif
 
 #define prod_h carry
@@ -1179,7 +1200,7 @@ vec_mul_small_on(int ulen, sc_digit *u, sc_digit v)
 
 #ifdef DEBUG_SYSTEMC
     // The overflow bits must be zero.
-    assert(high_half(u[i]) == high_half_masked(u[i]));
+    sc_assert(high_half(u[i]) == high_half_masked(u[i]));
 #endif
 
     sc_digit prod_l = v * low_half(u[i]) + low_half(carry);
@@ -1213,10 +1234,10 @@ vec_div_large(int ulen, const sc_digit *u,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
-  assert((vlen > 0) && (v != NULL));
-  assert(w != NULL);
-  assert(BITS_PER_DIGIT >= 3 * BITS_PER_BYTE);
+  sc_assert((ulen > 0) && (u != NULL));
+  sc_assert((vlen > 0) && (v != NULL));
+  sc_assert(w != NULL);
+  sc_assert(BITS_PER_DIGIT >= 3 * BITS_PER_BYTE);
 #endif
 
   // We will compute q = x / y where x = u and y = v. The reason for
@@ -1256,8 +1277,8 @@ vec_div_large(int ulen, const sc_digit *u,
   ylen++;
 
 #ifdef DEBUG_SYSTEMC
-  assert(xlen > 1);
-  assert(ylen > 1);
+  sc_assert(xlen > 1);
+  sc_assert(ylen > 1);
 #endif
 
   // At this point, all the leading zeros are eliminated from x and y.
@@ -1378,9 +1399,9 @@ vec_div_small(int ulen, const sc_digit *u,
   //        r = (((r|A) % v)|B) % v
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
-  assert(q != NULL);
-  assert((0 < v) && (v < HALF_DIGIT_RADIX));
+  sc_assert((ulen > 0) && (u != NULL));
+  sc_assert(q != NULL);
+  sc_assert((0 < v) && (v < HALF_DIGIT_RADIX));
 #endif
 
 #define q_h r
@@ -1397,7 +1418,7 @@ vec_div_small(int ulen, const sc_digit *u,
 
 #ifdef DEBUG_SYSTEMC
     // The overflow bits must be zero.
-    assert(high_half(u_AB) == high_half_masked(u_AB));
+    sc_assert(high_half(u_AB) == high_half_masked(u_AB));
 #endif
 
     sc_digit num = concat(r, high_half(u_AB));  // num = r|A
@@ -1421,10 +1442,10 @@ vec_rem_large(int ulen, const sc_digit *u,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
-  assert((vlen > 0) && (v != NULL));
-  assert(w != NULL);
-  assert(BITS_PER_DIGIT >= 3 * BITS_PER_BYTE);
+  sc_assert((ulen > 0) && (u != NULL));
+  sc_assert((vlen > 0) && (v != NULL));
+  sc_assert(w != NULL);
+  sc_assert(BITS_PER_DIGIT >= 3 * BITS_PER_BYTE);
 #endif
 
   // This function is adapted from vec_div_large.
@@ -1457,8 +1478,8 @@ vec_rem_large(int ulen, const sc_digit *u,
   ylen++;
 
 #ifdef DEBUG_SYSTEMC
-  assert(xlen > 1);
-  assert(ylen > 1);
+  sc_assert(xlen > 1);
+  sc_assert(ylen > 1);
 #endif
 
   // At this point, all the leading zeros are eliminated from x and y.
@@ -1549,8 +1570,8 @@ vec_rem_small(int ulen, const sc_digit *u, sc_digit v)
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
-  assert((0 < v) && (v < HALF_DIGIT_RADIX));
+  sc_assert((ulen > 0) && (u != NULL));
+  sc_assert((0 < v) && (v < HALF_DIGIT_RADIX));
 #endif
 
   // This function is adapted from vec_div_small().
@@ -1565,7 +1586,7 @@ vec_rem_small(int ulen, const sc_digit *u, sc_digit v)
 
 #ifdef DEBUG_SYSTEMC
     // The overflow bits must be zero.
-    assert(high_half(u_AB) == high_half_masked(u_AB));
+    sc_assert(high_half(u_AB) == high_half_masked(u_AB));
 #endif
 
     // r = (((r|A) % v)|B) % v
@@ -1582,8 +1603,8 @@ vec_rem_on_small(int ulen, sc_digit *u, sc_digit v)
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
-  assert(v > 0);
+  sc_assert((ulen > 0) && (u != NULL));
+  sc_assert(v > 0);
 #endif
 
 #define q_h r
@@ -1599,7 +1620,7 @@ vec_rem_on_small(int ulen, sc_digit *u, sc_digit v)
 
 #ifdef DEBUG_SYSTEMC
     // The overflow bits must be zero.
-    assert(high_half(u_AB) == high_half_masked(u_AB));
+    sc_assert(high_half(u_AB) == high_half_masked(u_AB));
 #endif
 
     sc_digit num = concat(r, high_half(u_AB));  // num = r|A
@@ -1623,8 +1644,8 @@ vec_to_char(int ulen, const sc_digit *u,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
-  assert((vlen > 0) && (v != NULL));
+  sc_assert((ulen > 0) && (u != NULL));
+  sc_assert((vlen > 0) && (v != NULL));
 #endif
 
   int nbits = ulen * BITS_PER_DIGIT;
@@ -1670,9 +1691,9 @@ vec_from_char(int ulen, const uchar *u,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
-  assert((vlen > 0) && (v != NULL));
-  assert(sizeof(uchar) <= sizeof(sc_digit));
+  sc_assert((ulen > 0) && (u != NULL));
+  sc_assert((vlen > 0) && (v != NULL));
+  sc_assert(sizeof(uchar) <= sizeof(sc_digit));
 #endif
 
   sc_digit *vend = (v + vlen);
@@ -1712,7 +1733,7 @@ vec_shift_left(int ulen, sc_digit *u, int nsl)
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
+  sc_assert((ulen > 0) && (u != NULL));
 #endif
 
   if (nsl <= 0)
@@ -1774,7 +1795,7 @@ vec_shift_right(int ulen, sc_digit *u, int nsr, sc_digit fill)
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((ulen > 0) && (u != NULL));
+  sc_assert((ulen > 0) && (u != NULL));
 #endif
 
   // fill is usually either 0 or DIGIT_MASK; it can be any value.
@@ -1842,16 +1863,16 @@ vec_reverse(int unb, int und, sc_digit *ud,
 {
 
 #ifdef DEBUG_SYSTEMC
-  assert((unb > 0) && (und > 0) && (ud != NULL));
-  assert((0 <= r) && (r <= l) && (l < unb));
+  sc_assert((unb > 0) && (und > 0) && (ud != NULL));
+  sc_assert((0 <= r) && (r <= l) && (l < unb));
 #endif
 
   if (l < r) {
-      char msg[BUFSIZ];
-      std::sprintf( msg, "vec_reverse( int, int, sc_digit*, int l, int r ) : "
-	       "l = %d < r = %d is not valid",
-	       l, r );
-      SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_, msg );
+      std::stringstream msg;
+      msg << "vec_reverse( int, int, sc_digit*, int l, int r ) : "
+          << "l = " << l << " < r = " << r << " is not valid",
+      SC_REPORT_ERROR( sc_core::SC_ID_CONVERSION_FAILED_, msg.str().c_str() );
+      return;
   }
 
   // Make sure that l and r are within bounds.
@@ -1883,9 +1904,19 @@ vec_reverse(int unb, int und, sc_digit *ud,
 #ifndef SC_MAX_NBITS
   delete [] d;
 #endif
-    
 }
 
+#ifdef SC_MAX_NBITS
+void test_bound_failed(int nb )
+{
+    std::stringstream msg;
+    msg << "test_bound( int nb ) : "
+           "nb = " << nb << " > SC_MAX_NBITS = " << SC_MAX_NBITS
+        << "  is not valid";
+    SC_REPORT_ERROR( sc_core::SC_ID_OUT_OF_BOUNDS_, msg.str().c_str() );
+}
+#endif // SC_MAX_NBITS
+
 } // namespace sc_dt
 
 
