@@ -34,7 +34,7 @@
 #if !defined(_WIN32) && !defined(WIN32) && !defined(WIN64)  && !defined(SC_USE_PTHREADS)
 
 #include "sysc/kernel/sc_cor.h"
-#include "sysc/qt/qt.h"
+#include "sysc/packages/qt/qt.h"
 
 namespace sc_core {
 
