@@ -26,8 +26,8 @@
  CHANGE LOG APPEARS AT THE END OF THE FILE
  *****************************************************************************/
 
-#include <stdlib.h>
-#include <string.h>
+#include <cstdlib>
+#include <cstring>
 
 #include "sysc/kernel/sc_event.h"
 #include "sysc/kernel/sc_kernel_ids.h"
@@ -38,14 +38,23 @@
 #include "sysc/kernel/sc_object_manager.h"
 #include "sysc/utils/sc_utils_ids.h"
 
+#include <sstream>
+
 namespace sc_core {
 
+using std::malloc;
+using std::strrchr;
+using std::strncmp;
+
 // ----------------------------------------------------------------------------
 //  CLASS : sc_event
 //
 //  The event class.
 // ----------------------------------------------------------------------------
 
+// kernel-internal event, that is never notified
+const sc_event sc_event::none( kernel_event, "none" );
+
 const char*
 sc_event::basename() const
 {
@@ -82,14 +91,11 @@ void
 sc_event::notify()
 {
     // immediate notification
-    if(
-        // coming from sc_prim_channel::update
-        m_simc->update_phase()
-#if SC_HAS_PHASE_CALLBACKS_
-        // coming from phase callbacks
-        || m_simc->notify_phase()
-#endif
-      )
+    if( !m_simc->evaluation_phase() )
+        // coming from
+        //  * elaboration
+        //  * sc_prim_channel::update
+        //  * phase callbacks
     {
         SC_REPORT_ERROR( SC_ID_IMMEDIATE_NOTIFICATION_, "" );
         return;
@@ -202,21 +208,24 @@ sc_event::notify_delayed( const sc_time& t )
     }
 }
 
+#define SC_KERNEL_EVENT_PREFIX "$$$$kernel_event$$$$_"
+sc_event::kernel_tag sc_event::kernel_event;
+
 // +----------------------------------------------------------------------------
 // |"sc_event::register_event"
-// | 
-// | This method sets the name of this object instance and optionally adds 
+// |
+// | This method sets the name of this object instance and optionally adds
 // | it to the object manager's hierarchy. The object instance will be
 // | inserted into the object manager's hierarchy if one of the following is
 // | true:
-// |   (a) the leaf name is non-null and does not start with  
-// |       SC_KERNEL_EVENT_PREFIX.
+// |   (a) the leaf name is non-null and is_kernel_event == false
 // |   (b) the event is being created before the start of simulation.
 // |
 // | Arguments:
 // |     leaf_name = leaf name of the object or NULL.
 // +----------------------------------------------------------------------------
-void sc_event::register_event( const char* leaf_name )
+void
+sc_event::register_event( const char* leaf_name, bool is_kernel_event /* = false */ )
 {
     sc_object_manager* object_manager = m_simc->get_object_manager();
     m_parent_p = m_simc->active_object();
@@ -225,8 +234,17 @@ void sc_event::register_event( const char* leaf_name )
 
     if( !leaf_name || !leaf_name[0] )
     {
-	if ( sc_is_running( m_simc ) ) return;
-        leaf_name = sc_gen_unique_name("event");    
+        if ( sc_is_running( m_simc ) ) return;
+        leaf_name = sc_gen_unique_name
+            ( is_kernel_event ? SC_KERNEL_EVENT_PREFIX : "event" );
+    }
+
+    // prepend kernel events with internal prefix
+    else if ( is_kernel_event )
+    {
+        m_name = SC_KERNEL_EVENT_PREFIX;
+        m_name.append( leaf_name );
+        leaf_name = m_name.c_str();
     }
 
     // Create a hierarchichal name and place it into the object manager if
@@ -234,8 +252,7 @@ void sc_event::register_event( const char* leaf_name )
 
     object_manager->create_name( leaf_name ).swap( m_name );
 
-    if ( strncmp( leaf_name, SC_KERNEL_EVENT_PREFIX, 
-                  strlen(SC_KERNEL_EVENT_PREFIX) ) )
+    if ( !is_kernel_event )
     {
 	object_manager->insert_event(m_name, this);
 	if ( m_parent_p )
@@ -259,9 +276,9 @@ sc_event::reset()
 
 // +----------------------------------------------------------------------------
 // |"sc_event::sc_event(name)"
-// | 
+// |
 // | This is the object instance constructor for named sc_event instances.
-// | If the name is non-null or the this is during elaboration add the
+// | If the name is non-null or this is during elaboration add the
 // | event to the object hierarchy.
 // |
 // | Arguments:
@@ -271,6 +288,7 @@ sc_event::sc_event( const char* name ) :
     m_name(),
     m_parent_p(NULL),
     m_simc( sc_get_curr_simcontext() ),
+    m_trigger_stamp( ~sc_dt::UINT64_ZERO ),
     m_notify_type( NONE ),
     m_delta_event_index( -1 ),
     m_timed( 0 ),
@@ -279,22 +297,21 @@ sc_event::sc_event( const char* name ) :
     m_threads_static(),
     m_threads_dynamic()
 {
-    // Skip simulator's internally defined events.
-
     register_event( name );
 }
 
 // +----------------------------------------------------------------------------
-// |"sc_event::sc_event(name)"
-// | 
+// |"sc_event::sc_event()"
+// |
 // | This is the object instance constructor for non-named sc_event instances.
-// | If this is during elaboration add create a name and add it to the object
+// | If this is during elaboration create a name and add it to the object
 // | hierarchy.
 // +----------------------------------------------------------------------------
 sc_event::sc_event() :
     m_name(),
     m_parent_p(NULL),
     m_simc( sc_get_curr_simcontext() ),
+    m_trigger_stamp( ~sc_dt::UINT64_ZERO ),
     m_notify_type( NONE ),
     m_delta_event_index( -1 ),
     m_timed( 0 ),
@@ -303,15 +320,37 @@ sc_event::sc_event() :
     m_threads_static(),
     m_threads_dynamic()
 {
-
     register_event( NULL );
 }
 
+// +----------------------------------------------------------------------------
+// |"sc_event::sc_event(kernel_event, name)"
+// |
+// | This is the object instance constructor for kernel sc_event instances.
+// | If this is during elaboration create an implementation-defined name and
+// | do NOT add it to the object hierarchy.
+// +----------------------------------------------------------------------------
+sc_event::sc_event( kernel_tag, const char* name ) :
+    m_name(),
+    m_parent_p(NULL),
+    m_simc( sc_get_curr_simcontext() ),
+    m_trigger_stamp( ~sc_dt::UINT64_ZERO ),
+    m_notify_type( NONE ),
+    m_delta_event_index( -1 ),
+    m_timed( 0 ),
+    m_methods_static(),
+    m_methods_dynamic(),
+    m_threads_static(),
+    m_threads_dynamic()
+{
+    register_event( name, /* is_kernel_event = */ true );
+}
+
 // +----------------------------------------------------------------------------
 // |"sc_event::~sc_event"
-// | 
+// |
 // | This is the object instance destructor for this class. It cancels any
-// | outstanding waits and removes the event from the object manager's 
+// | outstanding waits and removes the event from the object manager's
 // | instance table if it has a name.
 // +----------------------------------------------------------------------------
 sc_event::~sc_event()
@@ -322,25 +361,39 @@ sc_event::~sc_event()
 	sc_object_manager* object_manager_p = m_simc->get_object_manager();
 	object_manager_p->remove_event( m_name );
     }
+
+    for(size_t i = 0; i < m_threads_dynamic.size(); ++i ) {
+        if( m_threads_dynamic[i]->m_event_p == this )
+            m_threads_dynamic[i]->m_event_p = 0;
+    }
+    for(size_t i = 0; i < m_methods_dynamic.size(); ++i ) {
+        if( m_methods_dynamic[i]->m_event_p == this )
+            m_methods_dynamic[i]->m_event_p = 0;
+    }
 }
 
 // +----------------------------------------------------------------------------
 // |"sc_event::trigger"
-// | 
+// |
 // | This method "triggers" this object instance. This consists of scheduling
-// | for execution all the processes that are schedulable and waiting on this 
+// | for execution all the processes that are schedulable and waiting on this
 // | event.
 // +----------------------------------------------------------------------------
 void
 sc_event::trigger()
 {
+    m_trigger_stamp = m_simc->change_stamp();
+    m_notify_type = NONE;
+    m_delta_event_index = -1;
+    m_timed = 0;
+
     int       last_i; // index of last element in vector now accessing.
     int       size;   // size of vector now accessing.
 
 
     // trigger the static sensitive methods
 
-    if( ( size = m_methods_static.size() ) != 0 ) 
+    if( ( size = m_methods_static.size() ) != 0 )
     {
         sc_method_handle* l_methods_static = &m_methods_static[0];
         int i = size - 1;
@@ -353,7 +406,7 @@ sc_event::trigger()
     // trigger the dynamic sensitive methods
 
 
-    if( ( size = m_methods_dynamic.size() ) != 0 ) 
+    if( ( size = m_methods_dynamic.size() ) != 0 )
     {
 	last_i = size - 1;
 	sc_method_handle* l_methods_dynamic = &m_methods_dynamic[0];
@@ -373,7 +426,7 @@ sc_event::trigger()
 
     // trigger the static sensitive threads
 
-    if( ( size = m_threads_static.size() ) != 0 ) 
+    if( ( size = m_threads_static.size() ) != 0 )
     {
         sc_thread_handle* l_threads_static = &m_threads_static[0];
         int i = size - 1;
@@ -385,7 +438,7 @@ sc_event::trigger()
 
     // trigger the dynamic sensitive threads
 
-    if( ( size = m_threads_dynamic.size() ) != 0 ) 
+    if( ( size = m_threads_dynamic.size() ) != 0 )
     {
 	last_i = size - 1;
 	sc_thread_handle* l_threads_dynamic = &m_threads_dynamic[0];
@@ -401,12 +454,12 @@ sc_event::trigger()
 	}
         m_threads_dynamic.resize(last_i+1);
     }
-
-    m_notify_type = NONE;
-    m_delta_event_index = -1;
-    m_timed = 0;
 }
 
+bool sc_event::triggered() const
+{
+    return m_trigger_stamp == m_simc->change_stamp();
+}
 
 bool
 sc_event::remove_static( sc_method_handle method_h_ ) const
@@ -501,7 +554,7 @@ sc_event_timed::allocate()
 
     if( free_list == 0 ) {
         free_list = (sc_event_timed_u*) malloc( ALLOC_SIZE *
-                                                sizeof( sc_event_timed ) );
+                                                sizeof( sc_event_timed_u ) );
         int i = 0;
         for( ; i < ALLOC_SIZE - 1; ++ i ) {
             free_list[i].next = &free_list[i + 1];
@@ -518,7 +571,7 @@ void
 sc_event_timed::deallocate( void* p )
 {
     if( p != 0 ) {
-        sc_event_timed_u* q = RCAST<sc_event_timed_u*>( p );
+        sc_event_timed_u* q = reinterpret_cast<sc_event_timed_u*>( p );
         q->next = free_list;
         free_list = q;
     }
@@ -547,7 +600,7 @@ sc_event_list::push_back( const sc_event& e )
     m_events.push_back( &e );
 }
 
-void 
+void
 sc_event_list::push_back( const sc_event_list& el )
 {
     m_events.reserve( size() + el.size() );
@@ -630,17 +683,19 @@ sc_event_list::report_premature_destruction() const
     // is currently running (which is only part of the story):
 
     if( sc_get_current_process_handle().valid() ) {
-        // FIXME: improve error-handling
-        sc_assert( false && "sc_event_list prematurely destroyed" );
+        // called from a destructor, can't throw
+        SC_REPORT_FATAL( SC_ID_EVENT_LIST_FAILED_
+                       , "list prematurely destroyed" );
+        sc_abort();
     }
-
 }
 
 void
 sc_event_list::report_invalid_modification() const
 {
-    // FIXME: improve error-handling
-    sc_assert( false && "sc_event_list modfied while being waited on" );
+    SC_REPORT_ERROR( SC_ID_EVENT_LIST_FAILED_
+                   , "list modified while being waited on" );
+    // may continue, if suppressed
 }
 
 // ----------------------------------------------------------------------------
