@@ -41,9 +41,9 @@
 
 #ifndef SC_DEFINE_MESSAGE
 #define SC_DEFINE_MESSAGE(id,unused1,unused2) \
-    namespace sc_core { extern const char id[]; }
+    namespace sc_core { extern SC_API const char id[]; }
 namespace sc_core {
-    extern const char SC_ID_REGISTER_ID_FAILED_[]; // in sc_report_handler.cpp
+    extern SC_API const char SC_ID_REGISTER_ID_FAILED_[]; // in sc_report_handler.cpp
 }
 #endif
 
@@ -91,7 +91,7 @@ SC_DEFINE_MESSAGE(SC_ID_WAIT_NOT_ALLOWED_            , 519,
 SC_DEFINE_MESSAGE(SC_ID_NEXT_TRIGGER_NOT_ALLOWED_    , 520,
 	"next_trigger() is only allowed in SC_METHODs" )
 SC_DEFINE_MESSAGE(SC_ID_IMMEDIATE_NOTIFICATION_      , 521,
-	"immediate notification is not allowed during the update phase" )
+	"immediate notification is not allowed during update phase or elaboration" )
 SC_DEFINE_MESSAGE(SC_ID_HALT_NOT_ALLOWED_            , 522,
 	"halt() is only allowed in SC_CTHREADs" )
 SC_DEFINE_MESSAGE(SC_ID_WATCHING_NOT_ALLOWED_        , 523,
@@ -119,8 +119,10 @@ SC_DEFINE_MESSAGE(SC_ID_MODULE_NAME_STACK_EMPTY_     , 533,
 	"module name stack is empty: did you forget to "
 	"add a sc_module_name parameter to your module "
 	"constructor?" )
-// available message number 534
-// available message number 535
+SC_DEFINE_MESSAGE(SC_ID_NAME_EXISTS_                 , 534,
+	"name already exists")
+SC_DEFINE_MESSAGE( SC_ID_WAIT_NEGATIVE_CYCLES_       , 535,
+         "wait(n) not allowed for n <= 0" )
 SC_DEFINE_MESSAGE( SC_ID_IMMEDIATE_SELF_NOTIFICATION_, 536,
          "immediate self-notification ignored as of IEEE 1666-2011" )
 SC_DEFINE_MESSAGE( SC_ID_WAIT_DURING_UNWINDING_      , 537,
@@ -159,7 +161,8 @@ SC_DEFINE_MESSAGE(SC_ID_PHASE_CALLBACK_REGISTER_,        552,
        "register simulation phase callback" )
 SC_DEFINE_MESSAGE(SC_ID_PHASE_CALLBACK_FORBIDDEN_,       553,
        "forbidden action in simulation phase callback" )
-// available message number 554
+SC_DEFINE_MESSAGE( SC_ID_SIMULATION_START_UNEXPECTED_, 554,
+        "sc_start called unexpectedly" )
 // available message number 555
 SC_DEFINE_MESSAGE(SC_ID_THROW_IT_IGNORED_  , 556,
         "throw_it on method/non-running process is being ignored " )
@@ -177,12 +180,13 @@ SC_DEFINE_MESSAGE(SC_ID_NO_PROCESS_SEMANTICS_         , 563,
 	"Attempt to invoke process with no semantics() method" )
 SC_DEFINE_MESSAGE(SC_ID_EVENT_ON_NULL_PROCESS_         , 564,
 	"Attempt to get an event for non-existent process" )
-// available message number 565
+SC_DEFINE_MESSAGE(SC_ID_EVENT_LIST_FAILED_, 565,
+         "invalid use of sc_(and|or)_event_list" )
 SC_DEFINE_MESSAGE(SC_ID_UNKNOWN_PROCESS_TYPE_,       566,
 	"Unknown process type" )
-// available message number 567
-SC_DEFINE_MESSAGE(SC_ID_NEGATIVE_SIMULATION_TIME_, 568,
-        "negative simulation interval specified in sc_start call" )
+SC_DEFINE_MESSAGE(SC_ID_TIME_CONVERSION_FAILED_, 567,
+        "sc_time conversion failed")
+// available number 568
 SC_DEFINE_MESSAGE(SC_ID_BAD_SC_MODULE_CONSTRUCTOR_  , 569,
         "sc_module(const char*), sc_module(const std::string&) "
         "have been deprecated, use sc_module(const sc_module_name&)" )
