@@ -58,12 +58,13 @@
 //     P    = pointer to process message is for, or NULL in which case the
 //            message will not print.
 #if 0
+#   include <cstring>
 #   define DEBUG_NAME ""
 #   define DEBUG_MSG(NAME,P,MSG) \
     { \
-        if ( P && ( (strlen(NAME)==0) || !strcmp(NAME,P->name())) ) \
+        if ( P && ( (std::strlen(NAME)==0) || !std::strcmp(NAME,P->name())) ) \
           std::cout << "**** " << sc_time_stamp() << " ("  \
-	            << sc_get_current_process_name() << "): " << MSG \
+	            << sc_get_current_process_name("** NONE **") << "): " << MSG \
 		    << " - " << P->name() << std::endl; \
     }
 #else
@@ -77,7 +78,7 @@ namespace sc_core {
 
 void sc_method_cor_fn( void* );
 void sc_cmethod_cor_fn( void* );
-void sc_set_stack_size( sc_method_handle, std::size_t );
+SC_API void sc_set_stack_size( sc_method_handle, std::size_t );
 class sc_event;
 class sc_module;
 class sc_process_table;
@@ -85,14 +86,14 @@ class sc_process_handle;
 class sc_simcontext;
 class sc_runnable;
 
-void next_trigger( sc_simcontext* );
-void next_trigger( const sc_event&, sc_simcontext* );
-void next_trigger( const sc_event_or_list&, sc_simcontext* );
-void next_trigger( const sc_event_and_list&, sc_simcontext* );
-void next_trigger( const sc_time&, sc_simcontext* );
-void next_trigger( const sc_time&, const sc_event&, sc_simcontext* );
-void next_trigger( const sc_time&, const sc_event_or_list&, sc_simcontext* );
-void next_trigger( const sc_time&, const sc_event_and_list&, sc_simcontext* );
+SC_API void next_trigger( sc_simcontext* );
+SC_API void next_trigger( const sc_event&, sc_simcontext* );
+SC_API void next_trigger( const sc_event_or_list&, sc_simcontext* );
+SC_API void next_trigger( const sc_event_and_list&, sc_simcontext* );
+SC_API void next_trigger( const sc_time&, sc_simcontext* );
+SC_API void next_trigger( const sc_time&, const sc_event&, sc_simcontext* );
+SC_API void next_trigger( const sc_time&, const sc_event_or_list&, sc_simcontext* );
+SC_API void next_trigger( const sc_time&, const sc_event_and_list&, sc_simcontext* );
 
 struct sc_invoke_method; 
 //==============================================================================
