@@ -28,12 +28,13 @@
 
 
 #include <cassert>
-#include <math.h>
-#include <stdio.h>
+#include <cmath>
+#include <cstddef>
+#include <cstdio>
+#include <sstream>
 
 #include "sysc/kernel/sc_event.h"
 #include "sysc/kernel/sc_kernel_ids.h"
-#include "sysc/kernel/sc_macros_int.h"
 #include "sysc/kernel/sc_module.h"
 #include "sysc/kernel/sc_module_registry.h"
 #include "sysc/kernel/sc_name_gen.h"
@@ -42,6 +43,7 @@
 #include "sysc/kernel/sc_process_handle.h"
 #include "sysc/kernel/sc_simcontext.h"
 #include "sysc/kernel/sc_simcontext_int.h"
+#include "sysc/kernel/sc_object_int.h"
 #include "sysc/kernel/sc_reset.h"
 #include "sysc/communication/sc_communication_ids.h"
 #include "sysc/communication/sc_interface.h"
@@ -49,7 +51,6 @@
 #include "sysc/communication/sc_signal.h"
 #include "sysc/communication/sc_signal_ports.h"
 #include "sysc/utils/sc_utils_ids.h"
-#include "sysc/utils/sc_iostream.h"
 
 namespace sc_core {
 
@@ -98,7 +99,7 @@ sc_module_dynalloc_list::~sc_module_dynalloc_list()
 
 // ----------------------------------------------------------------------------
 
-sc_module*
+SC_API sc_module*
 sc_module_dynalloc( sc_module* module_ )
 {
     static sc_module_dynalloc_list dynalloc_list;
@@ -130,7 +131,7 @@ sc_bind_proxy::sc_bind_proxy( sc_port_base& port_ )
 {}
 
 
-const sc_bind_proxy SC_BIND_PROXY_NIL;
+SC_API const sc_bind_proxy SC_BIND_PROXY_NIL;
 
 
 // ----------------------------------------------------------------------------
@@ -148,7 +149,6 @@ sc_module::sc_module_init()
     m_module_name_p = 0;
     m_port_vec = new std::vector<sc_port_base*>;
     m_port_index = 0;
-    m_name_gen = new sc_name_gen;
 }
 
 /*
@@ -189,8 +189,10 @@ sc_module::sc_module()
        on the top of the stack */
     sc_module_name* mod_name = 
         simcontext()->get_object_manager()->top_of_module_name_stack();
-    if (0 == mod_name || 0 != mod_name->m_module_p)
+    if (0 == mod_name || 0 != mod_name->m_module_p) {
         SC_REPORT_ERROR( SC_ID_SC_MODULE_NAME_REQUIRED_, 0 );
+        sc_abort(); // can't recover from here
+    }
     sc_module_init();
     mod_name->set_module( this );
     m_module_name_p = mod_name; // must come after sc_module_init call.
@@ -217,8 +219,10 @@ sc_module::sc_module( const sc_module_name& )
        on the top of the stack */
     sc_module_name* mod_name = 
         simcontext()->get_object_manager()->top_of_module_name_stack();
-    if (0 == mod_name || 0 != mod_name->m_module_p)
-      SC_REPORT_ERROR( SC_ID_SC_MODULE_NAME_REQUIRED_, 0 );
+    if (0 == mod_name || 0 != mod_name->m_module_p) {
+        SC_REPORT_ERROR( SC_ID_SC_MODULE_NAME_REQUIRED_, 0 );
+        sc_abort(); // can't recover from here
+    }
     sc_module_init();
     mod_name->set_module( this );
     m_module_name_p = mod_name; // must come after sc_module_init call.
@@ -365,6 +369,7 @@ sc_module::reset_signal_is( const sc_signal_in_if<bool>& iface, bool level )
 const char*
 sc_module::gen_unique_name( const char* basename_, bool preserve_first )
 {
+    if( !m_name_gen ) m_name_gen = new sc_name_gen;
     return m_name_gen->gen_unique_name( basename_, preserve_first );
 }
 
@@ -381,9 +386,8 @@ sc_module::before_end_of_elaboration()
 void
 sc_module::construction_done()
 {
-    simcontext()->hierarchy_push( this );
+    hierarchy_scope scope(this);
     before_end_of_elaboration();
-    simcontext()->hierarchy_pop();
 }
 
 // called by elaboration_done (does nothing by default)
@@ -400,17 +404,16 @@ void
 sc_module::elaboration_done( bool& error_ )
 {
     if( ! m_end_module_called ) {
-	char msg[BUFSIZ];
-	std::sprintf( msg, "module '%s'", name() );
-	SC_REPORT_WARNING( SC_ID_END_MODULE_NOT_CALLED_, msg );
-	if( error_ ) {
-	    SC_REPORT_WARNING( SC_ID_HIER_NAME_INCORRECT_, 0 );
-	}
-	error_ = true;
+        std::stringstream msg;
+        msg << "module '" << name() << "'";
+        SC_REPORT_WARNING( SC_ID_END_MODULE_NOT_CALLED_, msg.str().c_str() );
+        if( error_ ) {
+            SC_REPORT_WARNING( SC_ID_HIER_NAME_INCORRECT_, 0 );
+        }
+        error_ = true;
     }
-    simcontext()->hierarchy_push( this );
+    hierarchy_scope scope(this);
     end_of_elaboration();
-    simcontext()->hierarchy_pop();
 }
 
 // called by start_simulation (does nothing by default)
@@ -422,9 +425,8 @@ sc_module::start_of_simulation()
 void
 sc_module::start_simulation()
 {
-    simcontext()->hierarchy_push( this );
+    hierarchy_scope scope(this);
     start_of_simulation();
-    simcontext()->hierarchy_pop();
 }
 
 // called by simulation_done (does nothing by default)
@@ -436,9 +438,8 @@ sc_module::end_of_simulation()
 void
 sc_module::simulation_done()
 {
-    simcontext()->hierarchy_push( this );
+    hierarchy_scope scope(this);
     end_of_simulation();
-    simcontext()->hierarchy_pop();
 }
 
 void
@@ -507,31 +508,32 @@ void
 sc_module::positional_bind( sc_interface& interface_ )
 {
     if( m_port_index == (int)m_port_vec->size() ) {
-	char msg[BUFSIZ];
-	if( m_port_index == 0 ) {
-	    std::sprintf( msg, "module `%s' has no ports", name() );
-	} else {
-	    std::sprintf( msg, "all ports of module `%s' are bound", name() );
-	}
-	SC_REPORT_ERROR( SC_ID_BIND_IF_TO_PORT_, msg );
+        std::stringstream msg;
+        if( m_port_index == 0 ) {
+            msg << "module `" << name() << "' has no ports";
+        } else {
+            msg << "all ports of module `" << name() << "' are bound";
+        }
+        SC_REPORT_ERROR( SC_ID_BIND_IF_TO_PORT_, msg.str().c_str() );
+        return;
     }
     int status = (*m_port_vec)[m_port_index]->pbind( interface_ );
     if( status != 0 ) {
-	char msg[BUFSIZ];
-	switch( status ) {
-	case 1:
-	    std::sprintf( msg, "port %d of module `%s' is already bound",
-		     m_port_index, name() );
-	    break;
-	case 2:
-	    std::sprintf( msg, "type mismatch on port %d of module `%s'",
-		     m_port_index, name() );
-	    break;
-	default:
-	    std::sprintf( msg, "unknown error" );
-	    break;
-	}
-	SC_REPORT_ERROR( SC_ID_BIND_IF_TO_PORT_, msg );
+        std::stringstream msg;
+        switch( status ) {
+        case 1:
+            msg << "port " << m_port_index
+                << " of module `" << name() << "' is already bound";
+            break;
+        case 2:
+            msg << "type mismatch on port " << m_port_index
+                << " of module `" << name() << "'";
+            break;
+        default:
+            msg << "unknown error";
+            break;
+        }
+        SC_REPORT_ERROR( SC_ID_BIND_IF_TO_PORT_, msg.str().c_str() );
     }
     ++ m_port_index;
 }
@@ -540,31 +542,32 @@ void
 sc_module::positional_bind( sc_port_base& port_ )
 {
     if( m_port_index == (int)m_port_vec->size() ) {
-	char msg[BUFSIZ];
-	if( m_port_index == 0 ) {
-	    std::sprintf( msg, "module `%s' has no ports", name() );
-	} else {
-	    std::sprintf( msg, "all ports of module `%s' are bound", name() );
-	}
-	SC_REPORT_ERROR( SC_ID_BIND_PORT_TO_PORT_, msg );
+        std::stringstream msg;
+        if( m_port_index == 0 ) {
+            msg << "module `" << name() << "' has no ports";
+        } else {
+            msg << "all ports of module `" << name() << "' are bound";
+        }
+        SC_REPORT_ERROR( SC_ID_BIND_IF_TO_PORT_, msg.str().c_str() );
+        return;
     }
     int status = (*m_port_vec)[m_port_index]->pbind( port_ );
     if( status != 0 ) {
-	char msg[BUFSIZ];
-	switch( status ) {
-	case 1:
-	    std::sprintf( msg, "port %d of module `%s' is already bound",
-		     m_port_index, name() );
-	    break;
-	case 2:
-	    std::sprintf( msg, "type mismatch on port %d of module `%s'",
-		     m_port_index, name() );
-	    break;
-	default:
-	    std::sprintf( msg, "unknown error" );
-	    break;
-	}
-	SC_REPORT_ERROR( SC_ID_BIND_PORT_TO_PORT_, msg );
+        std::stringstream msg;
+        switch( status ) {
+        case 1:
+            msg << "port " << m_port_index
+                << " of module `" << name() << "' is already bound";
+            break;
+        case 2:
+            msg << "type mismatch on port " << m_port_index
+                << " of module `" << name() << "'";
+            break;
+        default:
+            msg << "unknown error";
+            break;
+        }
+        SC_REPORT_ERROR( SC_ID_BIND_IF_TO_PORT_, msg.str().c_str() );
     }
     ++ m_port_index;
 }
@@ -651,7 +654,7 @@ sc_module::operator () ( const sc_bind_proxy& p001,
     {
         warn_only_once = false;
 	SC_REPORT_INFO(SC_ID_IEEE_1666_DEPRECATION_,
-	 "multiple () binding depreacted, use explicit port binding instead." );
+	 "multiple () binding deprecated, use explicit port binding instead." );
     }
 
     TRY_BIND( p001 );
