@@ -30,6 +30,7 @@
 #ifndef SC_VER_H
 #define SC_VER_H
 
+#include "sysc/kernel/sc_cmnhdr.h"
 #include "sysc/kernel/sc_macros.h"               // SC_CONCAT_UNDERSCORE_
                                                  // SC_STRINGIFY_HELPER_
 #include "sysc/communication/sc_writer_policy.h" // SC_DEFAULT_WRITER_POLICY
@@ -38,35 +39,35 @@
 
 namespace sc_core {
 
-extern const char* sc_copyright();
-extern const char* sc_release();
-extern const char* sc_version();
+extern SC_API const char* sc_copyright();
+extern SC_API const char* sc_release();
+extern SC_API const char* sc_version();
 
-extern const unsigned int sc_version_major;
-extern const unsigned int sc_version_minor;
-extern const unsigned int sc_version_patch;
+extern SC_API const unsigned int sc_version_major;
+extern SC_API const unsigned int sc_version_minor;
+extern SC_API const unsigned int sc_version_patch;
 
-extern const std::string  sc_version_originator;
-extern const std::string  sc_version_release_date;
-extern const std::string  sc_version_prerelease;
-extern const bool         sc_is_prerelease;
-extern const std::string  sc_version_string;
-extern const std::string  sc_copyright_string;
+extern SC_API const std::string  sc_version_originator;
+extern SC_API const std::string  sc_version_release_date;
+extern SC_API const std::string  sc_version_prerelease;
+extern SC_API const bool         sc_is_prerelease;
+extern SC_API const std::string  sc_version_string;
+extern SC_API const std::string  sc_copyright_string;
 
-#define SYSTEMC_2_3_1
- 
-#define SYSTEMC_VERSION       20140417
+#define SYSTEMC_2_3_3
+
+#define SYSTEMC_VERSION       20181013
 #define SC_VERSION_ORIGINATOR "Accellera"
 #define SC_VERSION_MAJOR      2
 #define SC_VERSION_MINOR      3
-#define SC_VERSION_PATCH      1
+#define SC_VERSION_PATCH      3
 #define SC_IS_PRERELEASE      0
 
 /// compliancy with IEEE 1666-2011 (see 8.6.5)
 #define IEEE_1666_SYSTEMC     201101L
 
 #define SC_COPYRIGHT                               \
-  "Copyright (c) 1996-2014 by all Contributors,\n" \
+  "Copyright (c) 1996-2018 by all Contributors,\n" \
   "ALL RIGHTS RESERVED\n"
 
 
@@ -93,12 +94,17 @@ extern const std::string  sc_copyright_string;
 // in it. That object instance will cause the constructor below
 // to be invoked. If the version of the SystemC being linked against
 // does not contain the constructor below a linkage error will occur.
+//
+// The static API check includes the SystemC version numbers as well as
+// the underlying C++ standard version (SC_CPLUSPLUS).
 
 #define SC_API_VERSION_STRING \
       SC_CONCAT_UNDERSCORE_( sc_api_version, \
       SC_CONCAT_UNDERSCORE_( SC_VERSION_MAJOR, \
       SC_CONCAT_UNDERSCORE_( SC_VERSION_MINOR, \
-                             SC_VERSION_PATCH ) ) )
+      SC_CONCAT_UNDERSCORE_( SC_VERSION_PATCH, \
+      SC_CONCAT_HELPER_( cxx, SC_CPLUSPLUS ) \
+  ) ) ) )
 
 // explicitly avoid macro expansion
 #define SC_API_DEFINED_( Symbol ) \
@@ -156,7 +162,14 @@ struct SC_API_VERSION_STRING
     );
 };
 
-#if !defined( SC_DISABLE_API_VERSION_CHECK ) // disabled in sc_ver.cpp
+#if !defined(SC_BUILD)
+// import explicitly instantiated template
+SC_TPLEXTERN_ template struct SC_API SC_API_VERSION_STRING
+<
+  &SC_DISABLE_VIRTUAL_BIND_CHECK_
+>;
+
+#if !defined(SC_DISABLE_API_VERSION_CHECK)
 static
 SC_API_VERSION_STRING
 <
@@ -168,6 +181,7 @@ api_version_check
   SC_DEFAULT_WRITER_POLICY
 );
 #endif // SC_DISABLE_API_VERSION_CHECK
+#endif // SC_BUILD
 
 //#undef SC_API_DEFINED_
 //#undef SC_API_UNDEFINED_
