@@ -46,12 +46,14 @@
 
  *****************************************************************************/
 
-#include <stdarg.h>
-#include <stdio.h>
+#include <cstdarg>
+#include <cstdio>
 
 #include "sysc/tracing/sc_trace.h"
 #include "sysc/tracing/sc_tracing_ids.h"
+
 #include "sysc/communication/sc_signal_ifs.h"
+#include "sysc/kernel/sc_event.h"
 #include "sysc/utils/sc_report.h"
 #include "sysc/utils/sc_utils_ids.h"
 
@@ -64,7 +66,7 @@ sc_trace_file::sc_trace_file()
   /* Intentionally blank */
 }
 
-void tprintf(sc_trace_file* tf,  const char* format, ...)
+void SC_API tprintf(sc_trace_file* tf,  const char* format, ...)
 {
     static char buffer[4096];
     va_list ap;
@@ -84,8 +86,13 @@ void sc_trace_file::delta_cycles(bool)
   /* Intentionally blank */
 }
 
+const sc_dt::uint64&
+sc_trace_file::event_trigger_stamp(const sc_event& ev) const
+{
+    return ev.m_trigger_stamp;
+}
 
-void
+SC_API void
 sc_trace( sc_trace_file* tf,
 	  const sc_signal_in_if<char>& object,
 	  const std::string& name,
@@ -96,7 +103,7 @@ sc_trace( sc_trace_file* tf,
     }
 }
 
-void
+SC_API void
 sc_trace( sc_trace_file* tf,
 	  const sc_signal_in_if<short>& object,
 	  const std::string& name,
@@ -107,7 +114,7 @@ sc_trace( sc_trace_file* tf,
     }
 }
 
-void
+SC_API void
 sc_trace( sc_trace_file* tf,
 	  const sc_signal_in_if<int>& object,
 	  const std::string& name,
@@ -118,7 +125,7 @@ sc_trace( sc_trace_file* tf,
     }
 }
 
-void
+SC_API void
 sc_trace( sc_trace_file* tf,
 	  const sc_signal_in_if<long>& object,
 	  const std::string& name,
@@ -130,7 +137,7 @@ sc_trace( sc_trace_file* tf,
 }
 
 
-void
+SC_API void
 sc_trace(sc_trace_file* /* not used */,
 	 const void* /* not used */,
 	 const std::string& name)
@@ -139,37 +146,23 @@ sc_trace(sc_trace_file* /* not used */,
 }
 
 
-
-void double_to_special_int64(double in, unsigned* high, unsigned* low)
-{
-    double invar = in;
-    if(invar > 5e17) invar = 5e17; // Saturation limit
-    if(invar < 0.0) invar = 0.0;
-    invar += .5;
-    *high = (unsigned)(invar / 1e9);
-    double rest = invar - 1e9 * (*high);
-    if(rest < 0) *low = 0;
-    else *low = (unsigned)rest;
-}
-
-
 // ----------------------------------------------------------------------------
 
 #define DEFN_TRACE_FUNC_REF_A(tp)                                             \
-void                                                                          \
-sc_trace( sc_trace_file* tf, const tp& object, const std::string& name ) \
+SC_API void                                                                   \
+sc_trace( sc_trace_file* tf, const tp& object, const std::string& name )      \
 {                                                                             \
     if( tf ) {                                                                \
-	tf->trace( object, name );                                            \
+	tf->trace( object, name );                                                \
     }                                                                         \
 }
 
 #define DEFN_TRACE_FUNC_PTR_A(tp)                                             \
-void                                                                          \
-sc_trace( sc_trace_file* tf, const tp* object, const std::string& name ) \
+SC_API void                                                                   \
+sc_trace( sc_trace_file* tf, const tp* object, const std::string& name )      \
 {                                                                             \
     if( tf ) {                                                                \
-	tf->trace( *object, name );                                           \
+	tf->trace( *object, name );                                               \
     }                                                                         \
 }
 
@@ -178,6 +171,50 @@ DEFN_TRACE_FUNC_REF_A(tp)                                                     \
 DEFN_TRACE_FUNC_PTR_A(tp)
 
 
+#define DEFN_TRACE_FUNC_REF_B(tp)                                             \
+SC_API void                                                                   \
+sc_trace( sc_trace_file* tf, const tp& object, const std::string& name,       \
+          int width)                                                          \
+{                                                                             \
+    if( tf ) {                                                                \
+	tf->trace( object, name, width );                                         \
+    }                                                                         \
+}
+
+#define DEFN_TRACE_FUNC_PTR_B(tp)                                             \
+SC_API void                                                                   \
+sc_trace( sc_trace_file* tf, const tp* object, const std::string& name,       \
+          int width)                                                          \
+{                                                                             \
+    if( tf ) {                                                                \
+	tf->trace( *object, name, width );                                        \
+    }                                                                         \
+}
+
+
+#define DEFN_TRACE_FUNC_B(tp)                                                 \
+DEFN_TRACE_FUNC_REF_B(tp)                                                     \
+DEFN_TRACE_FUNC_PTR_B(tp)
+
+
+DEFN_TRACE_FUNC_A( sc_event )
+DEFN_TRACE_FUNC_A( sc_time )
+
+DEFN_TRACE_FUNC_A( bool )
+DEFN_TRACE_FUNC_A( float )
+DEFN_TRACE_FUNC_A( double )
+
+DEFN_TRACE_FUNC_B( unsigned char )
+DEFN_TRACE_FUNC_B( unsigned short )
+DEFN_TRACE_FUNC_B( unsigned int )
+DEFN_TRACE_FUNC_B( unsigned long )
+DEFN_TRACE_FUNC_B( char )
+DEFN_TRACE_FUNC_B( short )
+DEFN_TRACE_FUNC_B( int )
+DEFN_TRACE_FUNC_B( long )
+DEFN_TRACE_FUNC_B( sc_dt::int64 )
+DEFN_TRACE_FUNC_B( sc_dt::uint64 )
+
 DEFN_TRACE_FUNC_A( sc_dt::sc_bit )
 DEFN_TRACE_FUNC_A( sc_dt::sc_logic )
 
@@ -186,16 +223,28 @@ DEFN_TRACE_FUNC_A( sc_dt::sc_uint_base )
 DEFN_TRACE_FUNC_A( sc_dt::sc_signed )
 DEFN_TRACE_FUNC_A( sc_dt::sc_unsigned )
 
-DEFN_TRACE_FUNC_REF_A( sc_dt::sc_bv_base )
-DEFN_TRACE_FUNC_REF_A( sc_dt::sc_lv_base )
+DEFN_TRACE_FUNC_A( sc_dt::sc_bv_base )
+DEFN_TRACE_FUNC_A( sc_dt::sc_lv_base )
 
+#ifdef SC_INCLUDE_FX
+
+DEFN_TRACE_FUNC_A( sc_dt::sc_fxval )
+DEFN_TRACE_FUNC_A( sc_dt::sc_fxval_fast )
+DEFN_TRACE_FUNC_A( sc_dt::sc_fxnum )
+DEFN_TRACE_FUNC_A( sc_dt::sc_fxnum_fast )
+
+#endif // SC_INCLUDE_FX
 
 #undef DEFN_TRACE_FUNC_REF_A
 #undef DEFN_TRACE_FUNC_PTR_A
 #undef DEFN_TRACE_FUNC_A
 
+#undef DEFN_TRACE_FUNC_REF_B
+#undef DEFN_TRACE_FUNC_PTR_B
+#undef DEFN_TRACE_FUNC_B
 
-void
+
+SC_API void
 sc_trace( sc_trace_file* tf,
 	  const unsigned int& object,
 	  const std::string& name,
