@@ -53,7 +53,9 @@
 
 namespace sc_core {
 
-class vcd_trace;  // defined in vcd_trace.cpp
+class sc_time;
+
+class vcd_trace;  // defined in sc_vcd_trace.cpp
 template<class T> class vcd_T_trace;
 
 
@@ -68,7 +70,7 @@ class vcd_trace_file
 {
 public:
 
-    enum vcd_enum {VCD_WIRE=0, VCD_REAL=1, VCD_LAST};
+    enum vcd_enum {VCD_WIRE=0, VCD_REAL, VCD_EVENT, VCD_TIME, VCD_LAST};
 
 	// sc_set_vcd_time_unit is deprecated.
 #if 0 // deprecated
@@ -88,30 +90,34 @@ protected:
     // These are all virtual functions in sc_trace_file and
     // they need to be defined here.
 
+    // Trace sc_time, sc_event
+    virtual void trace(const sc_time& object, const std::string& name);
+    virtual void trace(const sc_event& object, const std::string& name);
+
     // Trace a boolean object (single bit)
      void trace(const bool& object, const std::string& name);
 
     // Trace a sc_bit object (single bit)
-    virtual void trace( const sc_dt::sc_bit& object, 
+    virtual void trace( const sc_dt::sc_bit& object,
 	    const std::string& name);
 
     // Trace a sc_logic object (single bit)
      void trace(const sc_dt::sc_logic& object, const std::string& name);
-    
+
     // Trace an unsigned char with the given width
-     void trace(const unsigned char& object, const std::string& name, 
+     void trace(const unsigned char& object, const std::string& name,
      	int width);
 
     // Trace an unsigned short with the given width
-     void trace(const unsigned short& object, const std::string& name, 
+     void trace(const unsigned short& object, const std::string& name,
      	int width);
 
     // Trace an unsigned int with the given width
-     void trace(const unsigned int& object, const std::string& name, 
+     void trace(const unsigned int& object, const std::string& name,
      	int width);
 
     // Trace an unsigned long with the given width
-     void trace(const unsigned long& object, const std::string& name, 
+     void trace(const unsigned long& object, const std::string& name,
      	int width);
 
     // Trace a signed char with the given width
@@ -125,13 +131,13 @@ protected:
 
     // Trace a signed long with the given width
      void trace(const long& object, const std::string& name, int width);
-    
+
     // Trace an int64 with a given width
-     void trace(const sc_dt::int64& object, const std::string& name, 
+     void trace(const sc_dt::int64& object, const std::string& name,
          int width);
 
     // Trace a uint64 with a given width
-     void trace(const sc_dt::uint64& object, const std::string& name, 
+     void trace(const sc_dt::uint64& object, const std::string& name,
          int width);
 
     // Trace a float
@@ -141,15 +147,15 @@ protected:
      void trace(const double& object, const std::string& name);
 
     // Trace sc_dt::sc_uint_base
-     void trace (const sc_dt::sc_uint_base& object, 
+     void trace (const sc_dt::sc_uint_base& object,
 	 	const std::string& name);
 
     // Trace sc_dt::sc_int_base
-     void trace (const sc_dt::sc_int_base& object, 
+     void trace (const sc_dt::sc_int_base& object,
 	 	const std::string& name);
 
     // Trace sc_dt::sc_unsigned
-     void trace (const sc_dt::sc_unsigned& object, 
+     void trace (const sc_dt::sc_unsigned& object,
 	 	const std::string& name);
 
     // Trace sc_dt::sc_signed
@@ -159,18 +165,18 @@ protected:
     void trace( const sc_dt::sc_fxval& object, const std::string& name );
 
     // Trace sc_dt::sc_fxval_fast
-    void trace( const sc_dt::sc_fxval_fast& object, 
+    void trace( const sc_dt::sc_fxval_fast& object,
 		const std::string& name );
 
     // Trace sc_dt::sc_fxnum
     void trace( const sc_dt::sc_fxnum& object, const std::string& name );
 
     // Trace sc_dt::sc_fxnum_fast
-    void trace( const sc_dt::sc_fxnum_fast& object, 
+    void trace( const sc_dt::sc_fxnum_fast& object,
 		const std::string& name );
 
     template<class T>
-    void traceT(const T& object, const std::string& name, 
+    void traceT(const T& object, const std::string& name,
     	vcd_enum type=VCD_WIRE)
     {
         if( add_trace_check(name) )
@@ -179,16 +185,17 @@ protected:
     }
 
    // Trace sc_dt::sc_bv_base (sc_dt::sc_bv)
-    virtual void trace(const sc_dt::sc_bv_base& object, 
+    virtual void trace(const sc_dt::sc_bv_base& object,
 		const std::string& name);
 
     // Trace sc_dt::sc_lv_base (sc_dt::sc_lv)
-    virtual void trace(const sc_dt::sc_lv_base& object, 
+    virtual void trace(const sc_dt::sc_lv_base& object,
 	    const std::string& name);
+
     // Trace an enumerated object - where possible output the enumeration literals
     // in the trace file. Enum literals is a null terminated array of null
     // terminated char* literal strings.
-     void trace(const unsigned& object, const std::string& name, 
+     void trace(const unsigned& object, const std::string& name,
      	const char** enum_literals);
 
     // Output a comment to the trace file
@@ -199,18 +206,25 @@ protected:
 
 private:
 
+    template<typename T> const T& extract_ref(const T& object) const
+      { return object; }
+    const sc_dt::uint64& extract_ref(const sc_event& object) const
+      { return event_trigger_stamp(object); }
+
 #if SC_TRACING_PHASE_CALLBACKS_
     // avoid hidden overload warnings
-    virtual void trace( sc_trace_file* ) const { sc_assert(false); }
+    virtual void trace( sc_trace_file* ) const;
 #endif // SC_TRACING_PHASE_CALLBACKS_
 
     // Initialize the VCD tracing
     virtual void do_initialize();
+    void print_time_stamp(unit_type now_units_high, unit_type now_units_low) const;
+    bool get_time_stamp(unit_type &now_units_high, unit_type &now_units_low) const;
 
     unsigned vcd_name_index;           // Number of variables traced
 
-    unsigned previous_time_units_low;  // Previous time unit as 64-bit integer
-    unsigned previous_time_units_high;
+    unit_type previous_time_units_low;
+    unit_type previous_time_units_high;
 
 public:
 
