@@ -26,15 +26,21 @@
   CHANGE LOG AT END OF FILE
  *****************************************************************************/
 
-#include <assert.h>
-#include <ctype.h>
+#include <cctype>
 #include <cstdio>
-#include <stdarg.h>
-#include <string.h>
+#include <cstdarg>
+#include <cstring>
 
-#include "sysc/utils/sc_iostream.h"
 #include "sysc/utils/sc_string.h"
+#include "sysc/utils/sc_report.h"  // sc_assert
 #include "sysc/utils/sc_utils_ids.h"
+#include "sysc/utils/sc_report.h"
+
+using std::isspace;
+using std::strcmp;
+using std::strcpy;
+using std::strlen;
+using std::strncpy;
 
 namespace sc_dt {
 
@@ -51,7 +57,7 @@ sc_roundup( int n, int m )
 //  Reference counting string implementation class.
 // ----------------------------------------------------------------------------
 
-class sc_string_rep
+class SC_API sc_string_rep
 {
     friend class sc_string_old;
     friend ::std::ostream& operator<<( ::std::ostream&, const sc_string_old& );
@@ -80,7 +86,7 @@ class sc_string_rep
 
     ~sc_string_rep()
     {
-        assert( ref_count == 0 );
+        sc_assert( ref_count == 0 );
         delete[] str;
     }
 
@@ -374,7 +380,7 @@ sc_string_old::set( int i, char c )
 
 sc_string_old sc_string_old::to_string(const char* format, ...)
 {
-   va_list argptr;
+   std::va_list argptr;
    sc_string_old result;
    char buffer[1024]; // static string buffer
    buffer[1023]=000;
@@ -419,8 +425,10 @@ sc_string_old::print( ::std::ostream& os ) const
 
 void sc_string_old::test(int position)const
 {
-	if(position<0 || position>=length())
-		SC_REPORT_ERROR( sc_core::SC_ID_OUT_OF_BOUNDS_, "sc_string_old::test" );
+    if(position<0 || position>=length()) {
+        SC_REPORT_ERROR( sc_core::SC_ID_OUT_OF_BOUNDS_, "sc_string_old::test" );
+        sc_core::sc_abort(); // can't recover from here
+    }
 }
 
 // TODO: conveniece formatting functions for common types
@@ -500,8 +508,10 @@ sc_string_old::remove(unsigned index, unsigned length)
 sc_string_old&
 sc_string_old::insert(const sc_string_old& sub_string, unsigned index)
 {
-    if(index>(unsigned)length())   
-	SC_REPORT_ERROR( sc_core::SC_ID_OUT_OF_BOUNDS_, "sc_string_old::insert" );
+    if(index>(unsigned)length()) {
+        SC_REPORT_ERROR( sc_core::SC_ID_OUT_OF_BOUNDS_, "sc_string_old::insert" );
+        return *this;
+    }
     return (*this) = substr(0,index-1)+sub_string+substr(index,length()-1);
 }
 
