@@ -35,8 +35,8 @@
 #define SC_STRING_H
 
 
-#include "sysc/utils/sc_iostream.h"
 #include "sysc/utils/sc_report.h"
+#include <iostream>
 
 namespace sc_dt {
 	class sc_string_old;
@@ -64,10 +64,10 @@ sc_string_old operator + ( const char* s, const sc_string_old& t );
 //  String class (yet another).
 // ----------------------------------------------------------------------------
 
-class sc_string_old
+class SC_API sc_string_old
 {
-    friend systemc_ostream& operator << (systemc_ostream& os, const sc_string_old& a);
-    friend systemc_istream& operator >> ( systemc_istream& is, sc_string_old& a );
+    friend ::std::ostream& operator << (::std::ostream& os, const sc_string_old& a);
+    friend ::std::istream& operator >> ( ::std::istream& is, sc_string_old& a );
 
 public:
 
@@ -211,7 +211,7 @@ public:
     int cmp( const sc_string_old& s ) const;
 
 
-    void print( systemc_ostream& os = ::std::cout ) const;
+    void print( ::std::ostream& os = ::std::cout ) const;
 
 private:
 
@@ -227,8 +227,8 @@ private:
 // IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
 
 inline
-systemc_ostream&
-operator << ( systemc_ostream& os, const sc_string_old& a )
+::std::ostream&
+operator << ( ::std::ostream& os, const sc_string_old& a )
 {
     a.print( os );
     return os;
