@@ -24,16 +24,20 @@
 // 11-May-2011  John Aynsley  Add run-time check to release()
 
 
-#ifndef __TLM_GP_H__
-#define __TLM_GP_H__
+#ifndef TLM_CORE_TLM2_TLM_GP_H_INCLUDED_
+#define TLM_CORE_TLM2_TLM_GP_H_INCLUDED_
+
+#include "sysc/kernel/sc_cmnhdr.h" // SC_API
+#include "sysc/utils/sc_report.h" // sc_assert
+#include "sysc/datatypes/int/sc_nbdefs.h" // sc_dt::uint64
 
-#include <systemc>
 #include "tlm_core/tlm_2/tlm_generic_payload/tlm_array.h"
 
+#include <typeinfo> // std::type_info
+
 namespace tlm {
 
-class
-tlm_generic_payload;
+class tlm_generic_payload;
 
 class tlm_mm_interface {
 public:
@@ -42,19 +46,14 @@ public:
 };
 
 //---------------------------------------------------------------------------
-// Classes and helper functions for the extension mechanism
+// Classes and helpers for the extension mechanism
 //---------------------------------------------------------------------------
 // Helper function:
-inline unsigned int max_num_extensions(bool increment=false)
-{
-    static unsigned int max_num = 0;
-    if (increment) ++max_num;
-    return max_num;
-}
+SC_API unsigned int max_num_extensions();
 
 // This class can be used for storing pointers to the extension classes, used
 // in tlm_generic_payload:
-class tlm_extension_base
+class SC_API tlm_extension_base
 {
 public:
     virtual tlm_extension_base* clone() const = 0;
@@ -62,10 +61,7 @@ public:
     virtual void copy_from(tlm_extension_base const &) = 0;
 protected:
     virtual ~tlm_extension_base() {}
-    static unsigned int register_extension()
-    {
-        return (max_num_extensions(true) - 1);
-    };
+    static unsigned int register_extension(const std::type_info&);
 };
 
 // Base class for all extension classes, derive your extension class in
@@ -79,14 +75,14 @@ class tlm_extension : public tlm_extension_base
 {
 public:
     virtual tlm_extension_base* clone() const = 0;
-    virtual void copy_from(tlm_extension_base const &ext) = 0; //{assert(typeid(this)==typeid(ext)); assert(ID === ext.ID); assert(0);}
+    virtual void copy_from(tlm_extension_base const &ext) = 0;
     virtual ~tlm_extension() {}
     const static unsigned int ID;
 };
 
 template <typename T>
-const
-unsigned int tlm_extension<T>::ID = tlm_extension_base::register_extension();
+const unsigned int tlm_extension<T>::ID
+  = tlm_extension_base::register_extension(typeid(T));
 
 //---------------------------------------------------------------------------
 // enumeration types
@@ -119,7 +115,10 @@ enum tlm_gp_option {
 //---------------------------------------------------------------------------
 // The generic payload class:
 //---------------------------------------------------------------------------
-class tlm_generic_payload {
+
+SC_API_TEMPLATE_DECL_ tlm_array<tlm_extension_base*>;
+
+class SC_API tlm_generic_payload {
 
 public:
     //---------------
@@ -127,152 +126,32 @@ public:
     //---------------
 
     // Default constructor
-    tlm_generic_payload()
-        : m_address(0)
-        , m_command(TLM_IGNORE_COMMAND)
-        , m_data(0)
-        , m_length(0)
-        , m_response_status(TLM_INCOMPLETE_RESPONSE)
-        , m_dmi(false)
-        , m_byte_enable(0)
-        , m_byte_enable_length(0)
-        , m_streaming_width(0)
-        , m_gp_option(TLM_MIN_PAYLOAD)
-        , m_extensions(max_num_extensions())
-        , m_mm(0)
-        , m_ref_count(0)
-    {
+    tlm_generic_payload();
+    explicit tlm_generic_payload(tlm_mm_interface* mm);
+
+    void acquire() { sc_assert(m_mm != 0); m_ref_count++; }
+
+    void release() {
+        sc_assert(m_mm != 0 && m_ref_count > 0);
+        if (--m_ref_count==0)
+            m_mm->free(this);
     }
 
-    explicit tlm_generic_payload(tlm_mm_interface* mm)
-        : m_address(0)
-        , m_command(TLM_IGNORE_COMMAND)
-        , m_data(0)
-        , m_length(0)
-        , m_response_status(TLM_INCOMPLETE_RESPONSE)
-        , m_dmi(false)
-        , m_byte_enable(0)
-        , m_byte_enable_length(0)
-        , m_streaming_width(0)
-        , m_gp_option(TLM_MIN_PAYLOAD)
-        , m_extensions(max_num_extensions())
-        , m_mm(mm)
-        , m_ref_count(0)
-    {
-    }
+    int get_ref_count() const { return m_ref_count; }
 
-    void acquire(){assert(m_mm != 0); m_ref_count++;}
-    void release(){assert(m_mm != 0 && m_ref_count > 0); if (--m_ref_count==0) m_mm->free(this);}
-    int get_ref_count() const {return m_ref_count;}
     void set_mm(tlm_mm_interface* mm) { m_mm = mm; }
     bool has_mm() const { return m_mm != 0; }
 
-    void reset(){
-      //should the other members be reset too?
-      m_gp_option = TLM_MIN_PAYLOAD;
-      m_extensions.free_entire_cache();
-    };
-
+    void reset();
 
 private:
     //disabled copy ctor and assignment operator.
-    // Copy constructor
-    tlm_generic_payload(const tlm_generic_payload& x)
-        : m_address(x.get_address())
-        , m_command(x.get_command())
-        , m_data(x.get_data_ptr())
-        , m_length(x.get_data_length())
-        , m_response_status(x.get_response_status())
-        , m_dmi(x.is_dmi_allowed())
-        , m_byte_enable(x.get_byte_enable_ptr())
-        , m_byte_enable_length(x.get_byte_enable_length())
-        , m_streaming_width(x.get_streaming_width())
-        , m_gp_option(x.m_gp_option)
-        , m_extensions(max_num_extensions())
-    {
-        // copy all extensions
-        for(unsigned int i=0; i<m_extensions.size(); i++)
-        {
-            m_extensions[i] = x.get_extension(i);
-        }
-    }
+    tlm_generic_payload(const tlm_generic_payload& x) /* = delete */;
+    tlm_generic_payload& operator= (const tlm_generic_payload& x) /* = delete */;
 
-    // Assignment operator
-    tlm_generic_payload& operator= (const tlm_generic_payload& x)
-    {
-        m_command =            x.get_command();
-        m_address =            x.get_address();
-        m_data =               x.get_data_ptr();
-        m_length =             x.get_data_length();
-        m_response_status =    x.get_response_status();
-        m_byte_enable =        x.get_byte_enable_ptr();
-        m_byte_enable_length = x.get_byte_enable_length();
-        m_streaming_width =    x.get_streaming_width();
-        m_gp_option =          x.get_gp_option();
-        m_dmi =                x.is_dmi_allowed();
-
-        // extension copy: all extension arrays must be of equal size by
-        // construction (i.e. it must either be constructed after C++
-        // static construction time, or the resize_extensions() method must
-        // have been called prior to using the object)
-        for(unsigned int i=0; i<m_extensions.size(); i++)
-        {
-            m_extensions[i] = x.get_extension(i);
-        }
-        return (*this);
-    }
 public:
     // non-virtual deep-copying of the object
-    void deep_copy_from(const tlm_generic_payload & other)
-    {
-        m_command =            other.get_command();
-        m_address =            other.get_address();
-        m_length =             other.get_data_length();
-        m_response_status =    other.get_response_status();
-        m_byte_enable_length = other.get_byte_enable_length();
-        m_streaming_width =    other.get_streaming_width();
-        m_gp_option =          other.get_gp_option();
-        m_dmi =                other.is_dmi_allowed();
-
-        // deep copy data
-        // there must be enough space in the target transaction!
-        if(m_data && other.m_data)
-        {
-            memcpy(m_data, other.m_data, m_length);
-        }
-        // deep copy byte enables
-        // there must be enough space in the target transaction!
-        if(m_byte_enable && other.m_byte_enable)
-        {
-            memcpy(m_byte_enable, other.m_byte_enable, m_byte_enable_length);
-        }
-        // deep copy extensions (sticky and non-sticky)
-        for(unsigned int i=0; i<other.m_extensions.size(); i++)
-        {
-            if(other.m_extensions[i])
-            {                       //original has extension i
-                if(!m_extensions[i])
-                {                   //We don't: clone.
-                    tlm_extension_base *ext = other.m_extensions[i]->clone();
-                    if(ext)     //extension may not be clonable.
-                    {
-                        if(has_mm())
-                        {           //mm can take care of removing cloned extensions
-                            set_auto_extension(i, ext);
-                        }
-                        else
-                        {           // no mm, user will call free_all_extensions().
-                            set_extension(i, ext);
-                        }
-                    }
-                }
-                else
-                {                   //We already have such extension. Copy original over it.
-                    m_extensions[i]->copy_from(*other.m_extensions[i]);
-                }
-            }
-        }
-    }
+    void deep_copy_from(const tlm_generic_payload & other);
 
     // To update the state of the original generic payload from a deep copy
     // Assumes that "other" was created from the original by calling deep_copy_from
@@ -280,93 +159,18 @@ public:
     // when copying back the data array on a read command
 
     void update_original_from(const tlm_generic_payload & other,
-                              bool use_byte_enable_on_read = true)
-    {
-        // Copy back extensions that are present on the original
-        update_extensions_from(other);
+                              bool use_byte_enable_on_read = true);
 
-        // Copy back the response status and DMI hint attributes
-        m_response_status = other.get_response_status();
-        m_dmi             = other.is_dmi_allowed();
-
-        // Copy back the data array for a read command only
-        // deep_copy_from allowed null pointers, and so will we
-        // We assume the arrays are the same size
-        // We test for equal pointers in case the original and the copy share the same array
-
-        if(is_read() && m_data && other.m_data && m_data != other.m_data)
-        {
-            if (m_byte_enable && use_byte_enable_on_read)
-            {
-                if (m_byte_enable_length == 8 && m_length % 8 == 0 )
-                {
-                    // Optimized implementation copies 64-bit words by masking
-                    for (unsigned int i = 0; i < m_length; i += 8)
-                    {
-                        typedef sc_dt::uint64* u;
-                        *reinterpret_cast<u>(&m_data[i]) &= ~*reinterpret_cast<u>(m_byte_enable);
-                        *reinterpret_cast<u>(&m_data[i]) |= *reinterpret_cast<u>(&other.m_data[i]) &
-                                                            *reinterpret_cast<u>(m_byte_enable);
-                    }
-                }
-                else if (m_byte_enable_length == 4 && m_length % 4 == 0 )
-                {
-                    // Optimized implementation copies 32-bit words by masking
-                    for (unsigned int i = 0; i < m_length; i += 4)
-                    {
-                        typedef unsigned int* u;
-                        *reinterpret_cast<u>(&m_data[i]) &= ~*reinterpret_cast<u>(m_byte_enable);
-                        *reinterpret_cast<u>(&m_data[i]) |= *reinterpret_cast<u>(&other.m_data[i]) &
-                                                            *reinterpret_cast<u>(m_byte_enable);
-                    }
-                }
-                else
-                    // Unoptimized implementation
-                    for (unsigned int i = 0; i < m_length; i++)
-                        if ( m_byte_enable[i % m_byte_enable_length] )
-                            m_data[i] = other.m_data[i];
-            }
-            else
-              memcpy(m_data, other.m_data, m_length);
-        }
-    }
-
-    void update_extensions_from(const tlm_generic_payload & other)
-    {
-        // deep copy extensions that are already present
-        for(unsigned int i=0; i<other.m_extensions.size(); i++)
-        {
-            if(other.m_extensions[i])
-            {                       //original has extension i
-                if(m_extensions[i])
-                {                   //We have it too. copy.
-                    m_extensions[i]->copy_from(*other.m_extensions[i]);
-                }
-            }
-        }
-    }
+    void update_extensions_from(const tlm_generic_payload & other);
 
     // Free all extensions. Useful when reusing a cloned transaction that doesn't have memory manager.
     // normal and sticky extensions are freed and extension array cleared.
-    void free_all_extensions()
-    {
-        m_extensions.free_entire_cache();
-        for(unsigned int i=0; i<m_extensions.size(); i++)
-        {
-            if(m_extensions[i])
-            {
-                m_extensions[i]->free();
-                m_extensions[i] = 0;
-            }
-        }
-    }
+    void free_all_extensions();
+
     //--------------
     // Destructor
     //--------------
-    virtual ~tlm_generic_payload() {
-        for(unsigned int i=0; i<m_extensions.size(); i++)
-            if(m_extensions[i]) m_extensions[i]->free();
-    }
+    virtual ~tlm_generic_payload();
 
     //----------------
     // API (including setters & getters)
@@ -398,20 +202,7 @@ public:
     tlm_response_status  get_response_status() const {return m_response_status;}
     void                 set_response_status(const tlm_response_status response_status)
         {m_response_status = response_status;}
-    std::string          get_response_string() const
-    {
-        switch(m_response_status)
-        {
-        case TLM_OK_RESPONSE:            return "TLM_OK_RESPONSE";
-        case TLM_INCOMPLETE_RESPONSE:    return "TLM_INCOMPLETE_RESPONSE";
-        case TLM_GENERIC_ERROR_RESPONSE: return "TLM_GENERIC_ERROR_RESPONSE";
-        case TLM_ADDRESS_ERROR_RESPONSE: return "TLM_ADDRESS_ERROR_RESPONSE";
-        case TLM_COMMAND_ERROR_RESPONSE: return "TLM_COMMAND_ERROR_RESPONSE";
-        case TLM_BURST_ERROR_RESPONSE:   return "TLM_BURST_ERROR_RESPONSE";
-        case TLM_BYTE_ENABLE_ERROR_RESPONSE: return "TLM_BYTE_ENABLE_ERROR_RESPONSE";
-        }
-        return "TLM_UNKNOWN_RESPONSE";
-    }
+    std::string          get_response_string() const;
 
     // Streaming related methods
     unsigned int         get_streaming_width() const {return m_streaming_width;}
@@ -526,12 +317,7 @@ public:
 
     // non-templatized version with manual index:
     tlm_extension_base* set_extension(unsigned int index,
-                                      tlm_extension_base* ext)
-    {
-        tlm_extension_base* tmp = m_extensions[index];
-        m_extensions[index] = ext;
-        return tmp;
-    }
+                                      tlm_extension_base* ext);
 
     // Stick the pointer to an extension into the vector, return the
     // previous value and schedule its release
@@ -542,14 +328,7 @@ public:
 
     // non-templatized version with manual index:
     tlm_extension_base* set_auto_extension(unsigned int index,
-                                           tlm_extension_base* ext)
-    {
-        tlm_extension_base* tmp = m_extensions[index];
-        m_extensions[index] = ext;
-        if (!tmp) m_extensions.insert_in_cache(&m_extensions[index]);
-        assert(m_mm != 0);
-        return tmp;
-    }
+                                           tlm_extension_base* ext);
 
     // Check for an extension, ext will point to 0 if not present
     template <typename T> void get_extension(T*& ext) const
@@ -561,10 +340,7 @@ public:
         return static_cast<T*>(get_extension(T::ID));
     }
     // Non-templatized version with manual index:
-    tlm_extension_base* get_extension(unsigned int index) const
-    {
-        return m_extensions[index];
-    }
+    tlm_extension_base* get_extension(unsigned int index) const;
 
     //this call just removes the extension from the txn but does not
     // call free() or tells the MM to do so
@@ -602,23 +378,9 @@ public:
 
 private:
     // Non-templatized version with manual index
-    void clear_extension(unsigned int index)
-    {
-        m_extensions[index] = static_cast<tlm_extension_base*>(0);
-    }
+    void clear_extension(unsigned int index);
     // Non-templatized version with manual index
-    void release_extension(unsigned int index)
-    {
-        if (m_mm)
-        {
-            m_extensions.insert_in_cache(&m_extensions[index]);
-        }
-        else
-        {
-            m_extensions[index]->free();
-            m_extensions[index] = static_cast<tlm_extension_base*>(0);
-        }
-    }
+    void release_extension(unsigned int index);
 
 public:
     // Make sure the extension array is large enough. Can be called once by
@@ -626,10 +388,7 @@ public:
     // sure that the extension array is of correct size. This is only needed
     // if the initiator cannot guarantee that the generic payload object is
     // allocated after C++ static construction time.
-    void resize_extensions()
-    {
-        m_extensions.expand(max_num_extensions());
-    }
+    void resize_extensions();
 
 private:
     tlm_array<tlm_extension_base*> m_extensions;
@@ -639,4 +398,5 @@ private:
 
 } // namespace tlm
 
-#endif /* __TLM_GP_H__ */
+
+#endif /* TLM_CORE_TLM2_TLM_GP_H_INCLUDED_ */
