@@ -30,8 +30,8 @@
 // *****************************************************************************
 
 
-#ifndef __SIMPLE_TARGET_SOCKET_H__
-#define __SIMPLE_TARGET_SOCKET_H__
+#ifndef TLM_UTILS_SIMPLE_TARGET_SOCKET_H_INCLUDED_
+#define TLM_UTILS_SIMPLE_TARGET_SOCKET_H_INCLUDED_
 
 #ifndef SC_INCLUDE_DYNAMIC_PROCESSES // needed for sc_spawn
 #  define SC_INCLUDE_DYNAMIC_PROCESSES
@@ -39,16 +39,16 @@
 
 #include <systemc>
 #include <tlm>
+#include "tlm_utils/convenience_socket_bases.h"
 #include "tlm_utils/peq_with_get.h"
-#include <sstream>
 
 namespace tlm_utils {
 
-template <typename MODULE,
-          unsigned int BUSWIDTH = 32,
-          typename TYPES = tlm::tlm_base_protocol_types>
-class simple_target_socket :
-  public tlm::tlm_target_socket<BUSWIDTH, TYPES>
+template< typename MODULE, unsigned int BUSWIDTH, typename TYPES
+        , sc_core::sc_port_policy POL = sc_core::SC_ONE_OR_MORE_BOUND >
+class simple_target_socket_b
+  : public tlm::tlm_target_socket<BUSWIDTH, TYPES, 1, POL>
+  , protected simple_socket_base
 {
   friend class fw_process;
   friend class bw_process;
@@ -58,26 +58,21 @@ public:
   typedef tlm::tlm_sync_enum                            sync_enum_type;
   typedef tlm::tlm_fw_transport_if<TYPES>               fw_interface_type;
   typedef tlm::tlm_bw_transport_if<TYPES>               bw_interface_type;
-  typedef tlm::tlm_target_socket<BUSWIDTH, TYPES>       base_type;
+  typedef tlm::tlm_target_socket<BUSWIDTH,TYPES,1,POL>  base_type;
 
 public:
-  simple_target_socket() :
-    base_type(sc_core::sc_gen_unique_name("simple_target_socket")),
-    m_fw_process(this),
-    m_bw_process(this)
+  static const char* default_name()
+    { return sc_core::sc_gen_unique_name("simple_target_socket"); }
+
+  explicit simple_target_socket_b(const char* n = default_name())
+    : base_type(n)
+    , m_fw_process(this)
+    , m_bw_process(this)
   {
     bind(m_fw_process);
   }
 
-  explicit simple_target_socket(const char* n) :
-    base_type(n),
-    m_fw_process(this),
-    m_bw_process(this)
-  {
-    bind(m_fw_process);
-  }
-
-  using tlm::tlm_target_socket<BUSWIDTH, TYPES>::bind;
+  using base_type::bind;
 
   // bw transport must come thru us.
   tlm::tlm_bw_transport_if<TYPES> * operator ->() {return &m_bw_process;}
@@ -88,7 +83,7 @@ public:
                                                              phase_type&,
                                                              sc_core::sc_time&))
   {
-    assert(!sc_core::sc_get_curr_simcontext()->elaboration_done());
+    elaboration_check("register_nb_transport_fw");
     m_fw_process.set_nb_transport_ptr(mod, cb);
   }
 
@@ -96,14 +91,14 @@ public:
                             void (MODULE::*cb)(transaction_type&,
                                                sc_core::sc_time&))
   {
-    assert(!sc_core::sc_get_curr_simcontext()->elaboration_done());
+    elaboration_check("register_b_transport");
     m_fw_process.set_b_transport_ptr(mod, cb);
   }
 
   void register_transport_dbg(MODULE* mod,
                               unsigned int (MODULE::*cb)(transaction_type&))
   {
-    assert(!sc_core::sc_get_curr_simcontext()->elaboration_done());
+    elaboration_check("register_transport_dbg");
     m_fw_process.set_transport_dbg_ptr(mod, cb);
   }
 
@@ -111,10 +106,17 @@ public:
                                    bool (MODULE::*cb)(transaction_type&,
                                                       tlm::tlm_dmi&))
   {
-    assert(!sc_core::sc_get_curr_simcontext()->elaboration_done());
+    elaboration_check("register_get_direct_mem_ptr");
     m_fw_process.set_get_direct_mem_ptr(mod, cb);
   }
 
+protected:
+  void start_of_simulation()
+  {
+    base_type::start_of_simulation();
+    m_fw_process.start_of_simulation();
+  }
+
 private:
   //make call on bw path.
   sync_enum_type bw_nb_transport(transaction_type &trans, phase_type &phase, sc_core::sc_time &t)
@@ -132,39 +134,36 @@ private:
   class bw_process : public tlm::tlm_bw_transport_if<TYPES>
   {
   public:
-    bw_process(simple_target_socket *p_own) : m_owner(p_own)
+    bw_process(simple_target_socket_b *p_own) : m_owner(p_own)
     {
     }
 
     sync_enum_type nb_transport_bw(transaction_type &trans, phase_type &phase, sc_core::sc_time &t)
     {
-      typename std::map<transaction_type*, sc_core::sc_event *>::iterator it;
+      typename std::map<transaction_type*, sc_core::sc_event *>::iterator it =
+        m_owner->m_pending_trans.find(&trans);
 
-      it = m_owner->m_pending_trans.find(&trans);
       if(it == m_owner->m_pending_trans.end()) {
         // Not a blocking call, forward.
         return m_owner->bw_nb_transport(trans, phase, t);
 
-      } else {
-        if (phase == tlm::END_REQ) {
-          m_owner->m_end_request.notify(sc_core::SC_ZERO_TIME);
-          return tlm::TLM_ACCEPTED;
-
-        } else if (phase == tlm::BEGIN_RESP) {
-          if (m_owner->m_current_transaction == &trans) {
-            m_owner->m_end_request.notify(sc_core::SC_ZERO_TIME);
-          }
-          //TODO: add response-accept delay?
-          it->second->notify(t);
-          m_owner->m_pending_trans.erase(it);
-          return tlm::TLM_COMPLETED;
-
-        } else {
-          assert(0); exit(1);
-        }
-
-//        return tlm::TLM_COMPLETED;  //Should not reach here
       }
+
+      if (phase == tlm::END_REQ) {
+        m_owner->m_end_request.notify(sc_core::SC_ZERO_TIME);
+        return tlm::TLM_ACCEPTED;
+      }
+      if (phase == tlm::BEGIN_RESP) {
+        if (m_owner->m_current_transaction == &trans) {
+          m_owner->m_end_request.notify(sc_core::SC_ZERO_TIME);
+        }
+        //TODO: add response-accept delay?
+        it->second->notify(t);
+        m_owner->m_pending_trans.erase(it);
+        return tlm::TLM_COMPLETED;
+      }
+      m_owner->display_error("invalid phase received");
+      return tlm::TLM_COMPLETED;
     }
 
     void invalidate_direct_mem_ptr(sc_dt::uint64 s,sc_dt::uint64 e)
@@ -173,7 +172,7 @@ private:
     }
 
   private:
-    simple_target_socket *m_owner;
+    simple_target_socket_b *m_owner;
   };
 
   class fw_process : public tlm::tlm_fw_transport_if<TYPES>,
@@ -189,8 +188,7 @@ private:
     typedef bool (MODULE::*GetDirectMemPtr)(transaction_type&,
                                             tlm::tlm_dmi&);
 
-    fw_process(simple_target_socket *p_own) :
-      m_name(p_own->name()),
+    fw_process(simple_target_socket_b *p_own) :
       m_owner(p_own),
       m_mod(0),
       m_nb_transport_ptr(0),
@@ -199,63 +197,61 @@ private:
       m_get_direct_mem_ptr(0),
       m_peq(sc_core::sc_gen_unique_name("m_peq")),
       m_response_in_progress(false)
+    {}
+
+    void start_of_simulation()
     {
-      sc_core::sc_spawn_options opts;
-      opts.set_sensitivity(&m_peq.get_event());
-      sc_core::sc_spawn(sc_bind(&fw_process::b2nb_thread, this),
-                        sc_core::sc_gen_unique_name("b2nb_thread"), &opts);
+      if (!m_b_transport_ptr && m_nb_transport_ptr) { // only spawn b2nb_thread, if needed
+        sc_core::sc_spawn_options opts;
+        opts.set_sensitivity(&m_peq.get_event());
+        opts.dont_initialize();
+        sc_core::sc_spawn(sc_bind(&fw_process::b2nb_thread, this),
+                          sc_core::sc_gen_unique_name("b2nb_thread"), &opts);
+      }
     }
 
     void set_nb_transport_ptr(MODULE* mod, NBTransportPtr p)
     {
       if (m_nb_transport_ptr) {
-        std::stringstream s;
-        s << m_name << ": non-blocking callback allready registered";
-        SC_REPORT_WARNING("/OSCI_TLM-2/simple_socket",s.str().c_str());
-      } else {
-        assert(!m_mod || m_mod == mod);
-        m_mod = mod;
-        m_nb_transport_ptr = p;
+        m_owner->display_warning("non-blocking callback already registered");
+        return;
       }
+      sc_assert(!m_mod || m_mod == mod);
+      m_mod = mod;
+      m_nb_transport_ptr = p;
     }
 
     void set_b_transport_ptr(MODULE* mod, BTransportPtr p)
     {
       if (m_b_transport_ptr) {
-        std::stringstream s;
-        s << m_name << ": blocking callback allready registered";
-        SC_REPORT_WARNING("/OSCI_TLM-2/simple_socket",s.str().c_str());
-      } else {
-        assert(!m_mod || m_mod == mod);
-        m_mod = mod;
-        m_b_transport_ptr = p;
+        m_owner->display_warning("blocking callback already registered");
+        return;
       }
+      sc_assert(!m_mod || m_mod == mod);
+      m_mod = mod;
+      m_b_transport_ptr = p;
     }
 
     void set_transport_dbg_ptr(MODULE* mod, TransportDbgPtr p)
     {
       if (m_transport_dbg_ptr) {
-        std::stringstream s;
-        s << m_name << ": debug callback allready registered";
-        SC_REPORT_WARNING("/OSCI_TLM-2/simple_socket",s.str().c_str());
-      } else {
-        assert(!m_mod || m_mod == mod);
-        m_mod = mod;
-        m_transport_dbg_ptr = p;
+        m_owner->display_warning("debug callback already registered");
+        return;
       }
+      sc_assert(!m_mod || m_mod == mod);
+      m_mod = mod;
+      m_transport_dbg_ptr = p;
     }
 
     void set_get_direct_mem_ptr(MODULE* mod, GetDirectMemPtr p)
     {
       if (m_get_direct_mem_ptr) {
-        std::stringstream s;
-        s << m_name << ": get DMI pointer callback allready registered";
-        SC_REPORT_WARNING("/OSCI_TLM-2/simple_socket",s.str().c_str());
-      } else {
-        assert(!m_mod || m_mod == mod);
-        m_mod = mod;
-        m_get_direct_mem_ptr = p;
+        m_owner->display_warning("get DMI pointer callback already registered");
+        return;
       }
+      sc_assert(!m_mod || m_mod == mod);
+      m_mod = mod;
+      m_get_direct_mem_ptr = p;
     }
 // Interface implementation
     sync_enum_type nb_transport_fw(transaction_type& trans,
@@ -264,10 +260,12 @@ private:
     {
       if (m_nb_transport_ptr) {
         // forward call
-        assert(m_mod);
+        sc_assert(m_mod);
         return (m_mod->*m_nb_transport_ptr)(trans, phase, t);
+      }
 
-      } else if (m_b_transport_ptr) {
+      // nb->b conversion
+      if (m_b_transport_ptr) {
         if (phase == tlm::BEGIN_REQ) {
           // prepare thread to do blocking call
           process_handle_class * ph = m_process_handle.get_handle(&trans);
@@ -286,34 +284,30 @@ private:
 
           ph->m_e.notify(t);
           return tlm::TLM_ACCEPTED;
-
-        } else if (phase == tlm::END_RESP) {
+        }
+        if (phase == tlm::END_RESP) {
           m_response_in_progress = false;
           m_end_response.notify(t);
           return tlm::TLM_COMPLETED;
-
-        } else {
-          assert(0); exit(1);
-//          return tlm::TLM_COMPLETED;   ///< unreachable code
         }
-
-      } else {
-        std::stringstream s;
-        s << m_name << ": no non-blocking transport callback registered";
-        SC_REPORT_ERROR("/OSCI_TLM-2/simple_socket",s.str().c_str());
+        m_owner->display_error("invalid phase received");
+        return tlm::TLM_COMPLETED;
       }
-      return tlm::TLM_ACCEPTED;   ///< unreachable code
+      m_owner->display_error("no non-blocking transport callback registered");
+      return tlm::TLM_COMPLETED;
     }
 
     void b_transport(transaction_type& trans, sc_core::sc_time& t)
     {
       if (m_b_transport_ptr) {
         // forward call
-        assert(m_mod);
+        sc_assert(m_mod);
         (m_mod->*m_b_transport_ptr)(trans, t);
         return;
+      }
 
-      } else if (m_nb_transport_ptr) {
+      // b->nb conversion
+      if (m_nb_transport_ptr) {
         m_peq.notify(trans, t);
         t = sc_core::SC_ZERO_TIME;
 
@@ -339,25 +333,22 @@ private:
           }
           trans.set_mm(0);
         }
-
-      } else {
-        std::stringstream s;
-        s << m_name << ": no blocking transport callback registered";
-        SC_REPORT_ERROR("/OSCI_TLM-2/simple_socket",s.str().c_str());
+        return;
       }
+
+      // should not be reached
+      m_owner->display_error("no blocking transport callback registered");
     }
 
     unsigned int transport_dbg(transaction_type& trans)
     {
       if (m_transport_dbg_ptr) {
         // forward call
-        assert(m_mod);
+        sc_assert(m_mod);
         return (m_mod->*m_transport_dbg_ptr)(trans);
-
-      } else {
-        // No debug support
-        return 0;
       }
+      // No debug support
+      return 0;
     }
 
     bool get_direct_mem_ptr(transaction_type& trans,
@@ -365,16 +356,14 @@ private:
     {
       if (m_get_direct_mem_ptr) {
         // forward call
-        assert(m_mod);
+        sc_assert(m_mod);
         return (m_mod->*m_get_direct_mem_ptr)(trans, dmi_data);
-
-      } else {
-        // No DMI support
-        dmi_data.allow_read_write();
-        dmi_data.set_start_address(0x0);
-        dmi_data.set_end_address((sc_dt::uint64)-1);
-        return false;
       }
+      // No DMI support
+      dmi_data.allow_read_write();
+      dmi_data.set_start_address(0x0);
+      dmi_data.set_end_address((sc_dt::uint64)-1);
+      return false;
     }
 
   private:
@@ -435,7 +424,7 @@ private:
         sc_core::sc_time t = sc_core::SC_ZERO_TIME;
 
         // forward call
-        assert(m_mod);
+        sc_assert(m_mod);
         (m_mod->*m_b_transport_ptr)(*trans, t);
 
         sc_core::wait(t);
@@ -461,12 +450,10 @@ private:
     void b2nb_thread()
     {
       while (true) {
-        sc_core::wait(m_peq.get_event());
-
         transaction_type* trans;
         while ((trans = m_peq.get_next_transaction())!=0) {
-          assert(m_mod);
-          assert(m_nb_transport_ptr);
+          sc_assert(m_mod);
+          sc_assert(m_nb_transport_ptr);
           phase_type phase = tlm::BEGIN_REQ;
           sc_core::sc_time t = sc_core::SC_ZERO_TIME;
 
@@ -476,7 +463,7 @@ private:
             // notify transaction is finished
             typename std::map<transaction_type*, sc_core::sc_event *>::iterator it =
               m_owner->m_pending_trans.find(trans);
-            assert(it != m_owner->m_pending_trans.end());
+            sc_assert(it != m_owner->m_pending_trans.end());
             it->second->notify(t);
             m_owner->m_pending_trans.erase(it);
             break;
@@ -505,28 +492,29 @@ private:
               // notify transaction is finished
               typename std::map<transaction_type*, sc_core::sc_event *>::iterator it =
                 m_owner->m_pending_trans.find(trans);
-              assert(it != m_owner->m_pending_trans.end());
+              sc_assert(it != m_owner->m_pending_trans.end());
               it->second->notify(t);
               m_owner->m_pending_trans.erase(it);
               break;
             }
 
             default:
-              assert(0); exit(1);
-            };
+              m_owner->display_error("invalid phase received");
+            }
             break;
 
           default:
-            assert(0); exit(1);
-          };
+            m_owner->display_error("invalid sync value received");
+          }
         }
+        sc_core::wait();
       }
     }
 
     void free(tlm::tlm_generic_payload* trans)
     {
       mm_end_event_ext* ext = trans->template get_extension<mm_end_event_ext>();
-      assert(ext);
+      sc_assert(ext);
       // notif event first before freeing extensions (reset)
       ext->done.notify();
       trans->reset();
@@ -542,8 +530,7 @@ private:
     };
 
   private:
-    const std::string m_name;
-    simple_target_socket *m_owner;
+    simple_target_socket_b *m_owner;
     MODULE* m_mod;
     NBTransportPtr m_nb_transport_ptr;
     BTransportPtr m_b_transport_ptr;
@@ -554,6 +541,8 @@ private:
     sc_core::sc_event m_end_response;
   };
 
+private:
+  const sc_core::sc_object* get_socket() const { return this; }
 private:
   fw_process m_fw_process;
   bw_process m_bw_process;
@@ -562,12 +551,34 @@ private:
   transaction_type* m_current_transaction;
 };
 
+template< typename MODULE, unsigned int BUSWIDTH = 32
+        , typename TYPES = tlm::tlm_base_protocol_types >
+class simple_target_socket
+  : public simple_target_socket_b<MODULE,BUSWIDTH,TYPES>
+{
+  typedef simple_target_socket_b<MODULE,BUSWIDTH,TYPES> socket_b;
+public:
+  simple_target_socket() : socket_b() {}
+  explicit simple_target_socket(const char* name) : socket_b(name) {}
+};
+
+template< typename MODULE, unsigned int BUSWIDTH = 32
+        , typename TYPES = tlm::tlm_base_protocol_types >
+class simple_target_socket_optional
+  : public simple_target_socket_b<MODULE,BUSWIDTH,TYPES,sc_core::SC_ZERO_OR_MORE_BOUND>
+{
+  typedef simple_target_socket_b<MODULE,BUSWIDTH,TYPES,sc_core::SC_ZERO_OR_MORE_BOUND> socket_b;
+public:
+  simple_target_socket_optional() : socket_b() {}
+  explicit simple_target_socket_optional(const char* name) : socket_b(name) {}
+};
+
 //ID Tagged version
-template <typename MODULE,
-          unsigned int BUSWIDTH = 32,
-          typename TYPES = tlm::tlm_base_protocol_types>
-class simple_target_socket_tagged :
-  public tlm::tlm_target_socket<BUSWIDTH, TYPES>
+template< typename MODULE, unsigned int BUSWIDTH, typename TYPES
+        , sc_core::sc_port_policy POL = sc_core::SC_ONE_OR_MORE_BOUND >
+class simple_target_socket_tagged_b
+  : public tlm::tlm_target_socket<BUSWIDTH, TYPES, 1, POL>
+  , protected simple_socket_base
 {
   friend class fw_process;
   friend class bw_process;
@@ -577,26 +588,21 @@ public:
   typedef tlm::tlm_sync_enum                            sync_enum_type;
   typedef tlm::tlm_fw_transport_if<TYPES>               fw_interface_type;
   typedef tlm::tlm_bw_transport_if<TYPES>               bw_interface_type;
-  typedef tlm::tlm_target_socket<BUSWIDTH, TYPES>       base_type;
+  typedef tlm::tlm_target_socket<BUSWIDTH,TYPES,1,POL>  base_type;
 
 public:
-  simple_target_socket_tagged() :
-    base_type(sc_core::sc_gen_unique_name("simple_target_socket_tagged")),
-    m_fw_process(this),
-    m_bw_process(this)
+  static const char* default_name()
+    { return sc_core::sc_gen_unique_name("simple_target_socket_tagged"); }
+
+  explicit simple_target_socket_tagged_b(const char* n = default_name())
+    : base_type(n)
+    , m_fw_process(this)
+    , m_bw_process(this)
   {
     bind(m_fw_process);
   }
 
-  explicit simple_target_socket_tagged(const char* n) :
-    base_type(n),
-    m_fw_process(this),
-    m_bw_process(this)
-  {
-    bind(m_fw_process);
-  }
-
-  using tlm::tlm_target_socket<BUSWIDTH, TYPES>::bind;
+  using base_type::bind;
 
   // bw transport must come thru us.
   tlm::tlm_bw_transport_if<TYPES> * operator ->() {return &m_bw_process;}
@@ -609,7 +615,7 @@ public:
                                                              sc_core::sc_time&),
                                 int id)
   {
-    assert(!sc_core::sc_get_curr_simcontext()->elaboration_done());
+    elaboration_check("register_nb_transport_fw");
     m_fw_process.set_nb_transport_ptr(mod, cb);
     m_fw_process.set_nb_transport_user_id(id);
   }
@@ -620,7 +626,7 @@ public:
                                                sc_core::sc_time&),
                             int id)
   {
-    assert(!sc_core::sc_get_curr_simcontext()->elaboration_done());
+    elaboration_check("register_b_transport");
     m_fw_process.set_b_transport_ptr(mod, cb);
     m_fw_process.set_b_transport_user_id(id);
   }
@@ -630,7 +636,7 @@ public:
                                                          transaction_type&),
                               int id)
   {
-    assert(!sc_core::sc_get_curr_simcontext()->elaboration_done());
+    elaboration_check("register_transport_dbg");
     m_fw_process.set_transport_dbg_ptr(mod, cb);
     m_fw_process.set_transport_dbg_user_id(id);
   }
@@ -641,11 +647,18 @@ public:
                                                       tlm::tlm_dmi&),
                                    int id)
   {
-    assert(!sc_core::sc_get_curr_simcontext()->elaboration_done());
+    elaboration_check("register_get_direct_mem_ptr");
     m_fw_process.set_get_direct_mem_ptr(mod, cb);
     m_fw_process.set_get_dmi_user_id(id);
   }
 
+protected:
+  void start_of_simulation()
+  {
+    base_type::start_of_simulation();
+    m_fw_process.start_of_simulation();
+  }
+
 private:
   //make call on bw path.
   sync_enum_type bw_nb_transport(transaction_type &trans, phase_type &phase, sc_core::sc_time &t)
@@ -663,39 +676,34 @@ private:
   class bw_process : public tlm::tlm_bw_transport_if<TYPES>
   {
   public:
-    bw_process(simple_target_socket_tagged *p_own) : m_owner(p_own)
+    bw_process(simple_target_socket_tagged_b *p_own) : m_owner(p_own)
     {
     }
 
     sync_enum_type nb_transport_bw(transaction_type &trans, phase_type &phase, sc_core::sc_time &t)
     {
-      typename std::map<transaction_type*, sc_core::sc_event *>::iterator it;
+      typename std::map<transaction_type*, sc_core::sc_event *>::iterator it =
+        m_owner->m_pending_trans.find(&trans);
 
-      it = m_owner->m_pending_trans.find(&trans);
       if(it == m_owner->m_pending_trans.end()) {
         // Not a blocking call, forward.
         return m_owner->bw_nb_transport(trans, phase, t);
-
-      } else {
-        if (phase == tlm::END_REQ) {
-          m_owner->m_end_request.notify(sc_core::SC_ZERO_TIME);
-          return tlm::TLM_ACCEPTED;
-
-        } else if (phase == tlm::BEGIN_RESP) {
-          if (m_owner->m_current_transaction == &trans) {
-            m_owner->m_end_request.notify(sc_core::SC_ZERO_TIME);
-          }
-          //TODO: add response-accept delay?
-          it->second->notify(t);
-          m_owner->m_pending_trans.erase(it);
-          return tlm::TLM_COMPLETED;
-
-        } else {
-          assert(0); exit(1);
-        }
-
-//        return tlm::TLM_COMPLETED;  //Should not reach here
       }
+      if (phase == tlm::END_REQ) {
+        m_owner->m_end_request.notify(sc_core::SC_ZERO_TIME);
+        return tlm::TLM_ACCEPTED;
+      }
+      if (phase == tlm::BEGIN_RESP) {
+        if (m_owner->m_current_transaction == &trans) {
+          m_owner->m_end_request.notify(sc_core::SC_ZERO_TIME);
+        }
+        //TODO: add response-accept delay?
+        it->second->notify(t);
+        m_owner->m_pending_trans.erase(it);
+        return tlm::TLM_COMPLETED;
+      }
+      m_owner->display_error("invalid phase received");
+      return tlm::TLM_COMPLETED;
     }
 
     void invalidate_direct_mem_ptr(sc_dt::uint64 s,sc_dt::uint64 e)
@@ -704,7 +712,7 @@ private:
     }
 
   private:
-    simple_target_socket_tagged *m_owner;
+    simple_target_socket_tagged_b *m_owner;
   };
 
   class fw_process : public tlm::tlm_fw_transport_if<TYPES>,
@@ -724,8 +732,7 @@ private:
                                             transaction_type&,
                                             tlm::tlm_dmi&);
 
-    fw_process(simple_target_socket_tagged *p_own) :
-      m_name(p_own->name()),
+    fw_process(simple_target_socket_tagged_b *p_own) :
       m_owner(p_own),
       m_mod(0),
       m_nb_transport_ptr(0),
@@ -738,11 +745,17 @@ private:
       m_get_dmi_user_id(0),
       m_peq(sc_core::sc_gen_unique_name("m_peq")),
       m_response_in_progress(false)
+    {}
+
+    void start_of_simulation()
     {
-      sc_core::sc_spawn_options opts;
-      opts.set_sensitivity(&m_peq.get_event());
-      sc_core::sc_spawn(sc_bind(&fw_process::b2nb_thread, this),
-                        sc_core::sc_gen_unique_name("b2nb_thread"), &opts);
+      if (!m_b_transport_ptr && m_nb_transport_ptr) { // only spawn b2nb_thread, if needed
+        sc_core::sc_spawn_options opts;
+        opts.set_sensitivity(&m_peq.get_event());
+        opts.dont_initialize();
+        sc_core::sc_spawn(sc_bind(&fw_process::b2nb_thread, this),
+                          sc_core::sc_gen_unique_name("b2nb_thread"), &opts);
+      }
     }
 
     void set_nb_transport_user_id(int id) { m_nb_transport_user_id = id; }
@@ -753,53 +766,44 @@ private:
     void set_nb_transport_ptr(MODULE* mod, NBTransportPtr p)
     {
       if (m_nb_transport_ptr) {
-        std::stringstream s;
-        s << m_name << ": non-blocking callback allready registered";
-        SC_REPORT_WARNING("/OSCI_TLM-2/simple_socket",s.str().c_str());
-      } else {
-        assert(!m_mod || m_mod == mod);
-        m_mod = mod;
-        m_nb_transport_ptr = p;
+        m_owner->display_warning("non-blocking callback already registered");
+        return;
       }
+      sc_assert(!m_mod || m_mod == mod);
+      m_mod = mod;
+      m_nb_transport_ptr = p;
     }
 
     void set_b_transport_ptr(MODULE* mod, BTransportPtr p)
     {
       if (m_b_transport_ptr) {
-        std::stringstream s;
-        s << m_name << ": blocking callback allready registered";
-        SC_REPORT_WARNING("/OSCI_TLM-2/simple_socket",s.str().c_str());
-      } else {
-        assert(!m_mod || m_mod == mod);
-        m_mod = mod;
-        m_b_transport_ptr = p;
+        m_owner->display_warning("blocking callback already registered");
+        return;
       }
+      sc_assert(!m_mod || m_mod == mod);
+      m_mod = mod;
+      m_b_transport_ptr = p;
     }
 
     void set_transport_dbg_ptr(MODULE* mod, TransportDbgPtr p)
     {
       if (m_transport_dbg_ptr) {
-        std::stringstream s;
-        s << m_name << ": debug callback allready registered";
-        SC_REPORT_WARNING("/OSCI_TLM-2/simple_socket",s.str().c_str());
-      } else {
-        assert(!m_mod || m_mod == mod);
-        m_mod = mod;
-        m_transport_dbg_ptr = p;
+        m_owner->display_warning("debug callback already registered");
+        return;
       }
+      sc_assert(!m_mod || m_mod == mod);
+      m_mod = mod;
+      m_transport_dbg_ptr = p;
     }
 
     void set_get_direct_mem_ptr(MODULE* mod, GetDirectMemPtr p)
     {
       if (m_get_direct_mem_ptr) {
-        std::stringstream s;
-        s << m_name << ": get DMI pointer callback allready registered";
-        SC_REPORT_WARNING("/OSCI_TLM-2/simple_socket",s.str().c_str());
-      } else {
-        assert(!m_mod || m_mod == mod);
-        m_mod = mod;
-        m_get_direct_mem_ptr = p;
+        m_owner->display_warning("get DMI pointer callback already registered");
       }
+      sc_assert(!m_mod || m_mod == mod);
+      m_mod = mod;
+      m_get_direct_mem_ptr = p;
     }
 // Interface implementation
     sync_enum_type nb_transport_fw(transaction_type& trans,
@@ -808,10 +812,12 @@ private:
     {
       if (m_nb_transport_ptr) {
         // forward call
-        assert(m_mod);
+        sc_assert(m_mod);
         return (m_mod->*m_nb_transport_ptr)(m_nb_transport_user_id, trans, phase, t);
+      }
 
-      } else if (m_b_transport_ptr) {
+      // nb->b conversion
+      if (m_b_transport_ptr) {
         if (phase == tlm::BEGIN_REQ) {
 
           // prepare thread to do blocking call
@@ -831,34 +837,31 @@ private:
 
           ph->m_e.notify(t);
           return tlm::TLM_ACCEPTED;
-
-        } else if (phase == tlm::END_RESP) {
+        }
+        if (phase == tlm::END_RESP) {
           m_response_in_progress = false;
           m_end_response.notify(t);
           return tlm::TLM_COMPLETED;
-
-        } else {
-          assert(0); exit(1);
-//          return tlm::TLM_COMPLETED;   ///< unreachable code
         }
-
-      } else {
-        std::stringstream s;
-        s << m_name << ": no non-blocking transport callback registered";
-        SC_REPORT_ERROR("/OSCI_TLM-2/simple_socket",s.str().c_str());
+        m_owner->display_error("invalid phase");
+        return tlm::TLM_COMPLETED;
       }
-      return tlm::TLM_ACCEPTED;   ///< unreachable code
+
+      m_owner->display_error("no non-blocking transport callback registered");
+      return tlm::TLM_COMPLETED;
     }
 
     void b_transport(transaction_type& trans, sc_core::sc_time& t)
     {
       if (m_b_transport_ptr) {
         // forward call
-        assert(m_mod);
+        sc_assert(m_mod);
         (m_mod->*m_b_transport_ptr)(m_b_transport_user_id, trans, t);
         return;
+      }
 
-      } else if (m_nb_transport_ptr) {
+      // b->nb conversion
+      if (m_nb_transport_ptr) {
         m_peq.notify(trans, t);
         t = sc_core::SC_ZERO_TIME;
 
@@ -884,25 +887,21 @@ private:
           }
           trans.set_mm(0);
         }
-
-      } else {
-        std::stringstream s;
-        s << m_name << ": no transport callback registered";
-        SC_REPORT_ERROR("/OSCI_TLM-2/simple_socket",s.str().c_str());
+        return;
       }
+
+      m_owner->display_error("no transport callback registered");
     }
 
     unsigned int transport_dbg(transaction_type& trans)
     {
       if (m_transport_dbg_ptr) {
         // forward call
-        assert(m_mod);
+        sc_assert(m_mod);
         return (m_mod->*m_transport_dbg_ptr)(m_transport_dbg_user_id, trans);
-
-      } else {
-        // No debug support
-        return 0;
       }
+      // No debug support
+      return 0;
     }
 
     bool get_direct_mem_ptr(transaction_type& trans,
@@ -910,16 +909,14 @@ private:
     {
       if (m_get_direct_mem_ptr) {
         // forward call
-        assert(m_mod);
+        sc_assert(m_mod);
         return (m_mod->*m_get_direct_mem_ptr)(m_get_dmi_user_id, trans, dmi_data);
-
-      } else {
-        // No DMI support
-        dmi_data.allow_read_write();
-        dmi_data.set_start_address(0x0);
-        dmi_data.set_end_address((sc_dt::uint64)-1);
-        return false;
       }
+      // No DMI support
+      dmi_data.allow_read_write();
+      dmi_data.set_start_address(0x0);
+      dmi_data.set_end_address((sc_dt::uint64)-1);
+      return false;
     }
 
   private:
@@ -978,7 +975,7 @@ private:
         sc_core::sc_time t = sc_core::SC_ZERO_TIME;
 
         // forward call
-        assert(m_mod);
+        sc_assert(m_mod);
         (m_mod->*m_b_transport_ptr)(m_b_transport_user_id, *trans, t);
 
         sc_core::wait(t);
@@ -1004,12 +1001,10 @@ private:
     void b2nb_thread()
     {
       while (true) {
-        sc_core::wait(m_peq.get_event());
-
         transaction_type* trans;
         while ((trans = m_peq.get_next_transaction())!=0) {
-          assert(m_mod);
-          assert(m_nb_transport_ptr);
+          sc_assert(m_mod);
+          sc_assert(m_nb_transport_ptr);
           phase_type phase = tlm::BEGIN_REQ;
           sc_core::sc_time t = sc_core::SC_ZERO_TIME;
 
@@ -1019,7 +1014,7 @@ private:
             // notify transaction is finished
             typename std::map<transaction_type*, sc_core::sc_event *>::iterator it =
               m_owner->m_pending_trans.find(trans);
-            assert(it != m_owner->m_pending_trans.end());
+            sc_assert(it != m_owner->m_pending_trans.end());
             it->second->notify(t);
             m_owner->m_pending_trans.erase(it);
             break;
@@ -1048,28 +1043,29 @@ private:
               // notify transaction is finished
               typename std::map<transaction_type*, sc_core::sc_event *>::iterator it =
                 m_owner->m_pending_trans.find(trans);
-              assert(it != m_owner->m_pending_trans.end());
+              sc_assert(it != m_owner->m_pending_trans.end());
               it->second->notify(t);
               m_owner->m_pending_trans.erase(it);
               break;
             }
 
             default:
-              assert(0); exit(1);
+              m_owner->display_error("invalid phase received");
             };
             break;
 
           default:
-            assert(0); exit(1);
-          };
+            m_owner->display_error("invalid sync value received");
+          }
         }
+        sc_core::wait();
       }
     }
 
     void free(tlm::tlm_generic_payload* trans)
     {
       mm_end_event_ext* ext = trans->template get_extension<mm_end_event_ext>();
-      assert(ext);
+      sc_assert(ext);
       // notif event first before freeing extensions (reset)
       ext->done.notify();
       trans->reset();
@@ -1085,8 +1081,7 @@ private:
     };
 
   private:
-    const std::string m_name;
-    simple_target_socket_tagged *m_owner;
+    simple_target_socket_tagged_b *m_owner;
     MODULE* m_mod;
     NBTransportPtr m_nb_transport_ptr;
     BTransportPtr m_b_transport_ptr;
@@ -1101,6 +1096,8 @@ private:
     sc_core::sc_event m_end_response;
   };
 
+private:
+  const sc_core::sc_object* get_socket() const { return this; }
 private:
   fw_process m_fw_process;
   bw_process m_bw_process;
@@ -1109,6 +1106,27 @@ private:
   transaction_type* m_current_transaction;
 };
 
-}
+template< typename MODULE, unsigned int BUSWIDTH = 32
+        , typename TYPES = tlm::tlm_base_protocol_types >
+class simple_target_socket_tagged
+  : public simple_target_socket_tagged_b<MODULE,BUSWIDTH,TYPES>
+{
+  typedef simple_target_socket_tagged_b<MODULE,BUSWIDTH,TYPES> socket_b;
+public:
+  simple_target_socket_tagged() : socket_b() {}
+  explicit simple_target_socket_tagged(const char* name) : socket_b(name) {}
+};
 
-#endif
+template< typename MODULE, unsigned int BUSWIDTH = 32
+        , typename TYPES = tlm::tlm_base_protocol_types >
+class simple_target_socket_tagged_optional
+  : public simple_target_socket_tagged_b<MODULE,BUSWIDTH,TYPES,sc_core::SC_ZERO_OR_MORE_BOUND>
+{
+  typedef simple_target_socket_tagged_b<MODULE,BUSWIDTH,TYPES,sc_core::SC_ZERO_OR_MORE_BOUND> socket_b;
+public:
+  simple_target_socket_tagged_optional() : socket_b() {}
+  explicit simple_target_socket_tagged_optional(const char* name) : socket_b(name) {}
+};
+
+} // namespace tlm_utils
+#endif // TLM_UTILS_SIMPLE_TARGET_SOCKET_H_INCLUDED_
